#!/bin/bash
# will collect default 3 PM10/PM2.5 graphs period 365 days
# and join them into one graph
# and push the pictures to the website Behoud de Parel

# $Id: Create_Graphs.sh,v 1.27 2020/06/22 08:15:45 teus Exp teus $
# Copyright (C) 2015, Teus Hagen, the Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# MA2 04, HadM 06, Venray 131, Nettethal 066
declare -A TBL2NR NR2TBL SENS MAX
TBL2NR[HadM]=06	        ; NR2TBL[06]=HadM
TBL2NR[NETT]=066	; NR2TBL[066]=NETT
TBL2NR[NL10131]=131	; NR2TBL[131]=NL10131
TBL2NR[MA2]=04	        ; NR2TBL[04]=MA2
TBL2NR[MKH]=09	        ; NR2TBL[09]=MKH
TBL2NR[MHF]=07	        ; NR2TBL[07]=MHF
TBL2NR[GA]=03	        ; NR2TBL[03]=GA
STATIONS=${STATIONS:-06 131 066 04}
NORM=${NORM:-EU2010}	# use this norm in the graphs for limits
case "$NORM" in
EU2010|WHO|EU2008|AQI)
   export NORM=$NORM
   ;;
*)
   echo "Incorrect norm $NORM. Unknown." 1>&2
   exit 1
   ;;
esac
if [ -n "$1" ] ; then STATIONS="$*" ; fi
if [ -n "$VERBOSE" ] ; then export VERBOSE=1 ; fi
if [ -n "$DEBUG" ] ; then export DEBUG=1 ; fi
ENDDATE=${ENDDATE:-$(date "+%m-%d")}

# database access credentials
export DBUSER=${DBUSER}    # database user
export DBPASS=${DBPASS} # database password for access
export HOST=${HOST:-localhost}
export DBHOST=${DBHOST:-$HOST}
export DB=${DB:-luchtmetingen}

if which mysql_config_editor >/dev/null
then
    PASSED=$(mysql_config_editor print --login-path=$DB \
            | awk '/password =/{ print "HIDDEN" ; }')
    MYSQL="mysql --login-path=$DB -N -B --silent -h $DBHOST $DB"
else
    MYSQL="mysql -u $DBUSER -p$DBPASS -N -B --silent -h $DBHOST $DB"
fi

function GET_passwd() {
    local RTS=unknown
    if [ -n "$1" ] && [ $1 = USER ]
    then
        if [ -z "$DBUSER" ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --title "user name for $DB access" --entry --text "Select user:")
                echo $RTS
            else
                echo $DBUSER
            fi
        fi
    else
        if [ -z "$DBPASS" ] || [ ${PASSED:-0} = HIDDEN ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --password --title "${DBUSER:-UNKNOWN} password for $DB database access")
                echo $RTS
            else
                echo $DBPASS
            fi
        fi
    fi
    if [ -z "${RTS/unknown/}" ]
    then
        echo "$ERROR Need $DB database password or db user name (via env: DBUSER) for user ${DBUSER:-UNKNOWN}" 1>&2
        exit 1
    fi
}

if [ -z "${DBPASS}" ]
then
    DBPASS=""
    if which mysql_config_editor >/dev/null
    then
        PASSED=$(mysql_config_editor print --login-path=$DB \
           | awk '/password =/{ print "HIDDEN" ; }')
        MYSQL="mysql --login-path=$DB -N -B --silent -h $DBHOST $DB"
        DBUSER=""
    else
        DBPASS=$(GET_passwd)
        PASSED=0
        MYSQL="mysql -u $DBUSER -p$DBPASS -N -B --silent -h $DBHOST $DB"
    fi
    if [ ${PASSED:-0} != HIDDEN ] && [ -z "$DBPASS" ]
    then
        echo "$ERROR Need DB password!" 1>&2
        exit 1
    fi
fi
if [ -z "$DBUSER"  ] && [ "$PASSED" != HIDDEN ] ; then DBUSER=$(GET_passwd USER) ; fi

# Day_results.sh needs WWW Database access
export WWWDBUSER=${WWWDBUSER:-$DBUSER}	# not used
export WWWDBPASS=${WWWDBPASS:-$DBPASS}	# not used
export WWWDBHOST=${WWWDBHOST:-$DBHOST}	# not used
export WWWDB=${WWWDB:-parel7}	# not used

NAME=${NAME:-CrG_fijnstof}	# convert program does not allow @ in filenames!
SENS[131]="roet pm_25"
#SENS[131]="roet pm_25 pm_10 no2"
MAX[pm_10]="40 50"
MAX[pm_25]="20 25"
MAX[roet]="20 25"
CHECK="./Check_DB.sh"

# website location path
if [ -z "$WWW" ]
then
    for SITE in behouddeparel BdP
    do
        if [ -f /etc/apache2/sites-available/$SITE.conf ] ; then break ; fi
    done
    WWW=$(grep "DOCroot.*webdata" /etc/apache2/sites-available/$SITE.conf | head -1 | sed -e 's/.*DOCroot  *//' -e 's/\/ *$//' -e 's@/$@@')
    for DIR in sites/behouddeparel.nl cmsdata sites/default NULL
    do
        if [ -z "$WWW" ] ; then break ; fi
        if [ $DIR = NULL ] ; then WWW='' ; break ; fi
        DIR=$DIR/files/images
        if [ -d ${WWW}/$DIR ] ; then WWW=${WWW}/$DIR ; break ; fi
    done
fi
if [ -z "$WWW" -o ! -d ${WWW} -o ! -w ${WWW} ]
then
    echo "ERROR: Cannot find WWW dir $WWW." 1>&2
    exit 1
fi
# website owner
GRP=$(ls -ld "$WWW" | awk '{ print $4; }')

THUMP_WIDTH=150
# added with exiftool to graphs/diagrams images
# location of the HadM station
#GEOGPS="-GPSLongitudeRef=E -GPSLongitude=6.114165 -GPSLatitudeRef=N -GPSLatitude=51.4555119"

WDIR=/webdata/luchtmetingen
if [ ! -d $WDIR ]
then WDIR=$(pwd)
fi

if [ ! -d $WDIR -o ! -x $WDIR/$CHECK ]
then
    echo "$0: FATAL Cannot find $WDIR or $CHECK." 1>&2
    exit 1
fi

# geolocation of images, measurement station, is expected North, East
function Get_GPS() {
    local TYPE=${1,,}
    local VAL="$2";
    if [ -z "$TYPE" ] ; then return 1; fi
    local GPS=$($MYSQL -e "SELECT geolocation FROM stations WHERE stations.${TYPE} = '$VAL'")
    if [ -z "${GPS/NULL/}" ] ; then return 1; fi
    local GPSal=$($MYSQL -e "SELECT altitude FROM stations WHERE stations.${TYPE} = '$VAL'")
    GPSal=${GPSal/NULL/}
    if [ -n "$GPSal" ]
    then
        local AB=0
        if (( $GPSal < 0 )) ; then AB=1; GPSal=${GPSal/-/} ; fi
        GPSal=" -GPSAltitudeRef#=$AB -GPSAltitude=$GPSal "
    fi
    local GPSlo=${GPS/*N /}
    echo "-GPSLongitudeRef=E -GPSLongitude=${GPSlo/E/} -GPSLatitudeRef=N -GPSLatitude=${GPS/N*/} $GPSal"
}

#GEOGPS=$(Get_GPS nr 06)

function Create_Graph() {
    local PWD=$(pwd)
    local LOCATION=${1:-06}
    local GEOGPS=$(Get_GPS nr $LOCATION)
    cd $WDIR   
    local GRAPH
    local -i WIDTH
    local  -i YEAR
    local -i CNT=0 
    OUT=/var/tmp/"${NAME}_Y${YEARS:-3}_${NORM:-EU2010}-${LOCATION}.png"
    rm -f "$OUT"
    local HEIGHT
    for((YEAR=$(date "+%Y"); YEAR > $(date "+%Y") - ${YEARS:-3}; YEAR--))
    do
	if [ -n "$VERBOSE" ]
	then
	   echo "$CHECK NR=$LOCATION pollutant ${NORM:-EU2010} $YEAR-$ENDDATE" 1>&2
	fi
	GRAPH=$($CHECK NR=$LOCATION pollutant ${NORM:-EU2010} $YEAR-$ENDDATE 2>&1 | grep OUTPUT | sed -e 's/.*.* \//\//' -e 's/\.png.*/.png/')
	if [ ! -f "$GRAPH" ]
	then
	    echo "No full year RRD graph for location $LOCATION before $YEAR-$ENDDATE"
	    break
	fi
	if [ ! -f /var/tmp/legenda.$$.png ]
        then
	    HEIGHT=$(file "$GRAPH" | sed -e 's/.*PNG image data,  *[0-9][0-9]*  *x  *//' -e 's/,  *.*//')
	    convert -quiet $GRAPH -crop ${WIDTH}x$((${HEIGHT-168}))+0+168 +repage /var/tmp/legenda.$$.png
	fi
	WIDTH=$(file "$GRAPH" | sed -e 's/.*PNG image data,  *//' -e 's/  *x  *.*//')
	if [ -z "$WIDTH" -o "$WIDTH" -lt 500 ]
	then
	    echo "Incorrect file $GRAPH, RRD graph location $LOCATION for $YEAR-$ENDDATE" 1>&2
	    break
	fi
	if [ ! -f "$OUT" ]
	then
	    cp "$GRAPH" "$OUT"
	else	# we have to join this one with previous graph
	    if ! convert -quiet "$OUT" -crop ${WIDTH}x$((168 * $CNT))+0+0 +repage "${NAME}-$YEAR.png"
	    then
		echo "Warning: cannot crop the image $OUT $YEAR location $LOCATION" 1>&2
		rm -f "${NAME}-$YEAR.png" "$GRAPH" "${GRAPH}_orginal"
		continue
	    fi
	    rm -f "$OUT"
	    convert -quiet "${NAME}-$YEAR.png" "$GRAPH" -append "$OUT"
	    rm -f "${NAME}-$YEAR.png"
	fi
	rm -f "$GRAPH"
	CNT+=1
    done
    if [ ! -f "$OUT" ]
    then
	cd $PWD
	echo "ERROR: no $OUT file created." 1>&2
	return 1
    fi
    if [ -f /var/tmp/legenda.$$.png ]
    then
        convert -quiet "$OUT" -crop ${WIDTH}x$((168 * $CNT))+0+0 +repage "/var/tmp/sav.$$.png"
        convert -quiet "/var/tmp/sav.$$.png" "/var/tmp/legenda.$$.png" -append "$OUT"
        rm -f /var/tmp/legenda.$$.png "/var/tmp/sav.$$.png"
    fi
    if [ -f "$OUT" ]
    then
	local PREV=$(date --date="last year" "+%Y")
	local COMMENT=$(exiftool -Comment "$OUT" | sed -e 's/Comment  *: //' -e "s/$PREV/$YEAR/")
	local TITLE=$(exiftool -Title "$OUT" | sed -e 's/Title  *: //' -e "s/$PREV/$YEAR/")
        exiftool -q -m -overwrite_original_in_place \
	 $GEOGPS \
         -Title="$TITLE" \
         -Comment="$COMMENT, norm used in color grading: ${NORM:-EU2010}" \
         "$OUT" >/dev/null
        rm -f "${OUT}_original"
	convert "$OUT" -filter spline -resize "${THUMP_WIDTH}x" -unsharp 0x0.75+0.75+0.008 "${OUT/.png/-thumbnail.png}"
	local NEWnme=$(basename "${OUT/CrG_/@}")
	if [ -d "$WWW" -a -z "$DEBUG" ]
	then
	    cp "$OUT" "$WWW/${NEWnme}"
	    cp "${OUT/.png/-thumbnail.png}" "$WWW/${NEWnme/.png/-thumbnail.png}"
	    rm -f "$OUT" "${OUT/.png/-thumbnail.png}"
	    chgrp $GRP "$WWW/${NEWnme}" "$WWW/${NEWnme/.png/-thumbnail.png}"
            echo "Copied new $NEWnme graph and thumbnail  into: $WWW"
        else
            echo "Created graphs: $OUT and ${OUT/.png/-thumbnail.png}."
	fi
	cd $PWD
	return 0
    fi
    cd $PWD
    return 1
}

# take care: convert does not like filenames starting with @
# at the website we use @fijnstof... names.
function Create_Sensor_Graph() {
    local PWD=$(pwd)
    local LOCATION=${1:-131}
    local SENSOR="${2:-roet}"
    declare -i LAST=365	# default max is one year
    local F
    local SENSORS=()
    for F in $SENSOR
    do
	declare -i FI
        FI=$($MYSQL -e "
	SELECT datediff(now(),datum) FROM ${NR2TBL[$LOCATION]} WHERE NOT isnull($F) limit 1;")
        if [ $FI -lt 7 ] ; then continue ; fi
	if [ $FI -lt $LAST ] ; then LAST=$FI ; fi
	SENSORS+=("$F")
    done
    if [ ${#SENSORS[@]} -le 0 ]
    then
	echo "WARNING: no sensors found." 1>&2 ; return 1
    fi
    local GEOGPS=$(Get_GPS nr $LOCATION)
    cd $WDIR   
    local GRAPH
    local -i WIDTH
    local  -i YEAR
    local -i CNT=0 
    # OUT="@sensors_Y${YEARS:-3}_${NORM:-EU2010}-${LOCATION}.png"
    OUT="CrG_sensors_Y${YEARS:-3}-${LOCATION}.png"
    rm -f "$OUT"
    local HEIGHT
    F=$LAST
    for((YEAR=$(date "+%Y"); YEAR > $(date "+%Y") - ${YEARS:-3}; YEAR--))
    do
	if [ $LAST -le 0 ] ; then break ; fi
	if [ $F -lt 365 ]
	then
	    F=$LAST
	else
	    F=365
        fi
	LAST=$(($LAST - 365))
	if [ -n "$VERBOSE" ]
	then
	   echo "$CHECK NR=$LOCATION sensor ${SENSORS[@]} $F $YEAR-$ENDDATE" 1>&2
	fi
	# GRAPH=$($CHECK NR=$LOCATION sensor ${SENSORS[@]} $F $YEAR-$ENDDATE 2>&1 | grep OUTPUT | sed -e 's/.*file:  *//' -e 's/\.png.*/.png/')
	GRAPH=$($CHECK NR=$LOCATION sensor ${SENSORS[@]} $F $YEAR-$ENDDATE 2>&1 | grep OUTPUT | sed -e 's/.* \//\//' -e 's/\.png.*/.png/')
	if [ ! -f "$GRAPH" ]
	then
	    echo "Failed to collect RRD graph location $LOCATION over $F days until $YEAR-$ENDDATE"
	    continue
	fi
	if [ ! -f /var/tmp/legenda.$$.png ]
        then
	    HEIGHT=$(file "$GRAPH" | sed -e 's/.*PNG image data,  *[0-9][0-9]*  *x  *//' -e 's/,  *.*//')
	    convert -quiet $GRAPH -crop ${WIDTH}x$((${HEIGHT-168}))+0+168 +repage /var/tmp/legenda.$$.png
	fi
	WIDTH=$(file "$GRAPH" | sed -e 's/.*PNG image data,  *//' -e 's/  *x  *.*//')
	if [ -z "$WIDTH" -o "$WIDTH" -lt 500 ]
	then
	    echo "Incorrect file $GRAPH, RRD graph location $LOCATION for $YEAR-$ENDDATE" 1>&2
	    continue
	fi
	if [ ! -f "$OUT" ]
	then
	    cp "$GRAPH" "$OUT"
	else	# we have to join this one with previous graph
	    if ! convert -quiet "$OUT" -crop ${WIDTH}x$((168 * $CNT))+0+0 +repage "${NAME}-$YEAR.png"
	    then
		echo "Warning: cannot crop the image $OUT $YEAR location $LOCATION" 1>&2
		rm -f "${NAME}-$YEAR.png" "$GRAPH" "${GRAPH}_orginal"
		continue
	    fi
	    rm -f "$OUT"
	    convert -quiet "${NAME}-$YEAR.png" "$GRAPH" -append "$OUT"
	    rm -f "${NAME}-$YEAR.png"
	fi
	rm -f "$GRAPH"
	if [ -z "$F" ] ; then break ; fi
	CNT+=1
    done
    convert -quiet "$OUT" -crop ${WIDTH}x$((168 * $CNT))+0+0 +repage "sav.$OUT"
    convert -quiet "sav.$OUT" "/var/tmp/legenda.$$.png" -append "$OUT"
    rm -f /var/tmp/legenda.$$.png "sav.$OUT"
    if [ -f "$OUT" ]
    then
	local PREV=$(date --date="last year" "+%Y")
	local COMMENT=$(exiftool -Comment "$OUT" | sed -e 's/Comment  *: //' -e "s/$PREV/$YEAR/")
	local TITLE=$(exiftool -Title "$OUT" | sed -e 's/Title  *: //' -e "s/$PREV/$YEAR/")
        exiftool -q -m -overwrite_original_in_place \
	 $GEOGPS \
         -Title="$SENSOR $TITLE" \
         -Comment="Sensors: $SENSOR, $COMMENT, norm color grading ${NORM:-EU2010}" \
         "$OUT" >/dev/null
        rm -f "${OUT}_original"
	convert "$OUT" -filter spline -resize "${THUMP_WIDTH}x" -unsharp 0x0.75+0.75+0.008 "${OUT/.png/-thumbnail.png}"
	# @ problem handling in file names with convert:
	local NEWnme=$(basename "${OUT/CrG_/@}")
	if [ -d "$WWW" -a -z "$DEBUG" ]
	then
	    cp "$OUT" "$WWW/${NEWnme}"
	    cp "${OUT/.png/-thumbnail.png}" "$WWW/${NEWnme/.png/-thumbnail.png}"
	    chgrp $GRP "$WWW/$NEWnme" "$WWW/${NEWnme/.png/-thumbnail.png}"
	    rm -f "$OUT" "${OUT/.png/-thumbnail.png}"
	fi
        echo "Created graphs: $NEWnme and ${NEWnme/.png/-thumbnail.png}."
	cd $PWD
	return 0
    fi
    cd $PWD
    return 1
}

if [ -n "$1" -a "$1" == "help" ]
then
   cat >/dev/stderr <<EOF
$0: Create graphs from RRD database for stations:
Maastricht A2:     04
Horst aan de Maas: 06
Venray Vredepeel: 131
Nettethal (Dld):  066
Graphs and stumbnail graphs will be pushed to the website dir $WWW/${NAME/CrG_/@}_($NORM}-{station_nr}[-thumbnail].png
Environment variable NORM=${NORM:-EU2010} will define emission standard used, default EU2010
Other supported standards are: EU2008, WHO, AQI.
DEBUG=1 pushes graphs to current directory
WWW=path sets WWW directory to push the graphs into
DB{USER,PASS,HOST,DB}= will define the ${DB:-luchtmetingen} DB credentials
WWWDB{USER,PASS,HOST,DB}= will define the ${WWWDB:-parel7} website DB credentials

Usage: [NORM=${NORM:-EU2010}] $0 station_nr ...
EOF
    exit 0
fi

for LOC in $STATIONS
do
    case $LOC in
    [A-Za-z]*)
	LOC=${TBL2NR[$LOC]}
    ;;
    esac
    if [ -z "$LOC" ] ; then continue; fi
    if [ -n "${SENS[$LOC]}" ]
    then
        Create_Sensor_Graph "$LOC" "${SENS[$LOC]}"
    fi
    Create_Graph "$LOC"
done
