#!/bin/bash
# GPL, Teus Hagen Apr 2015
#
# script created a post-it with to day news for particular matter

# $Id: Day_results.sh,v 4.8 2019/11/20 11:11:48 teus Exp teus $
# Copyright (C) 2015, Teus Hagen, the Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# script depends on post-it script to generate post-it image from $POSTIT SVG template
# depends on convert


DEBUG=${DEBUG}			# do not delete tempfile
VERBOSE=${VERBOSE}		# do not print overview
if [ -n "$VERBOSE" ] ; then VERBOSE=/dev/stdout ; else VERBOSE=/dev/null ; fi

# luchtmetingen database credentials
export DBUSER=${DBUSER}	# database user
export DBPASS=${DBPASS} # database password for access
HOST=${HOST:-localhost}
export DBHOST=${DBHOST:-$HOST}
export DB=${DB:-luchtmetingen}
# website DB credentials
export WWWDBUSER=${WWWDBUSER:-$DBUSER}
# WWWDBPASS see below
export WWWDBHOST=${WWWDBHOST:-$DBHOST}
export WWWDB=${WWWDB:-parel7}

# script to update html website article
METINGEN=/webdata/luchtmetingen/metingen.pl
# include forecast into dagwaarden.html
FORECAST='-fFORECAST'        # disabled if empty

ALIAS=dagwaardenTEMPLATE

TBLS=${TBLS:-HadM NETT NL10131 MA2}

NO_POL='(id|datum|_[a-z]+|[A-Z]+)'      # name is not pollutant, use with grep -E

# perl routine to obtain air quality index values for a range of standards
INDEXSCRIPT=/webdata/luchtmetingen/AQI.pl

function Usage() {
    echo "
	Get last day measurements PM10 and PM2.5 from some measurements
	stations as (default): TBLS=\"${TBLS}\" (env. variable)
	Command optional argument(s):
		[KENMERKEN] station ID ...
		Overview [station ID] report status station
		Movies	collect PM10 + PM2.5 movies for NL, push them to website
	Without arguments only the post-it and alerts will be produced.
	If last day PM10 measurements is above SMOG or dayly level (EU2010)
	push a post-it to website and send email to $SMOG_EMAIL and $FIJN_EMAIL
	Argument maybe one or stations: create a short report only.

	Calculate last 365 days dayly average and report excess of level.
	With only stations as arguments the website page $ALIAS alias will
	be updated and posted via ${METINGEN} script.

	If first argument is KENMERKEN the day measurement data will be
	produced in KENMERKEN.data file. This data can be used in ${METINGEN}.
        If KENMERKEN is used the ${METINGEN} command is skipped.

	${METINGEN} -H KENMERKEN.data 3911 temp (output goes into temp.html)
	where 3911 is node number of $ALIAS in database $DB.

	Environment variables:
            The KEY=\"$KEY\" (dflt: -aqi -lki -caqi -aqhi -H) will be options
            for the ${METINGEN} command.

	    DB{USER,PASS,DB,HOST} credentials for DB access.
	    WWWDB{USER,PASS,DB,HOST} credentials for website DB access.
	    e.g. DBUSER=${DBUSER:-user} DB=${DB:-metingenDB} DBPASS=${DBPASS:-XXX}
	    WWWDBUSER=${WWWDBUSER:-DrupalUser} WWWDBPASS=${WWWDBPASS:-DrupalPass}
	    WWWDBHOST=${WWWDBHOST:-localhost} DBHOST=${DBHOST:-localhost}
	    WWWDB=${WWWDB:-DrupalDB}.

	    VERBOSE (print full overview output on stdout as well)
	    DEBUG (do not delete tempfile)
"
}
if [ -n "$1" -a "$1" = help ]
then
	Usage
	exit 0
fi
	

MYSQL="mysql -u $DBUSER -p$DBPASS -N -B --silent -h $DBHOST $DB"
if which mysql_config_editor >/dev/null
then
    PASSED=$(mysql_config_editor print --login-path=$DB \
        | awk '/password =/{ print "HIDDEN" ; }')
    MYSQL="mysql --login-path=$DB -N -B --silent -h $DBHOST $DB"
fi

# get the pollutants from the DB table arg1
function Get_pollutants() {
    if [ -z "$1" ] ; then return 1 ; fi
    $MYSQL -e "DESCRIBE $1" | perl -e "while(<STDIN>) { s/\\s.*//; print \$_ if not /${NO_POL}/ ; }"
    return $?
}   

function GET_passwd() {
    local RTS=unknown
    if [ -n "$1" ] && [ $1 = USER ]
    then
        if [ -z "$DBUSER" ] && [ {PASSED:-0} = HIDDEN ]
        then
            export DBUSER=$(mysql_config_editor print --login-path=$DB \
                | awk '/user =/{ print $3 ; }')
        fi
        if [ -z "${DBHOST/localhost/}" ] && [ {PASSED:-0} = HIDDEN ]
        then
            export DBHOST=$(mysql_config_editor print --login-path=$DB \
                | awk '/host =/{ print $3 ; }')
        fi
        if [ -z "$DBUSER" ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --title "user name for $DB access" --entry --text "Select user:")
                echo $RTS
            else
                echo $DBUSER
            fi
        fi
    else
        if [ -z "$DBPASS" ] || [ ${PASSED:-0} = HIDDEN ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --password --title "${DBUSER:-UNKNOWN} password for $DB database access")
                echo $RTS
            else
                echo $DBPASS
            fi
        fi
    fi
    if [ -z "${RTS/unknown/}" ]
    then
        echo "$ERROR Need $DB database password or db user name (via env: DBUSER) for user ${DBUSER:-UNKNOWN}" 1>&2
        exit 1
    fi
}

if [ -z "$DBPASS" ] ; then DBPASS=$(GET_passwd) ; fi

# the NODE number of overview per day in DB WWWDB is obtained via alias

# website DB credentials
export WWWDBPASS=${WWWDBPASS:-$DBPASS}
WWWMYSQL="mysql -u $WWWDBUSER -p$WWWDBPASS -N -B --silent -h $WWWDBHOST $WWWDB"
if echo "$MYSQL" | grep -q login-path
then
    if [ "$DBUSER" = "$WWWDBUSER" ]
    then
        WWWMYSQL="mysql --login-path=$DB -N -B --silent -h $WWWDBHOST $WWWDB"
    else
        WWWMYSQL="mysql --login-path=$WWWDB -N -B --silent -h $WWWDBHOST $WWWDB"
    fi
fi

# -H output is in HTML format optional add file name
KEY="${KEY:--lki -aqi -aqhi -caqi} -H"  # -aqi -aqhi adds AQI index, -lki LKI index

export LC_TIME="nl_NL.UTF-8"	# set to NL for time strings

# geolocation of images, station HadM
function Get_GPS() {
    local TYPE=${1,,}
    local VAL="$2";
    if [ -z "$TYPE" ] ; then return 1; fi
    local GPS=$($MYSQL -e "SELECT geolocation FROM stations WHERE stations.${TYPE} = '$VAL'")
    if [ -z "${GPS/NULL/}" ] ; then return 1; fi
    local GPSal=$($MYSQL -e "SELECT altitude FROM stations WHERE stations.${TYPE} = '$VAL'")
    GPSal=${GPSal/NULL/}
    if [ -n "$GPSal" ]
    then
        local AB=0
        if (( $GPSal < 0 )) ; then AB=1; GPSal=${GPSal/-/} ; fi
        GPSal=" -GPSAltitudeRef#=$AB -GPSAltitude=$GPSal "
    fi
    local GPSlo=${GPS/*N /}
    echo "-GPSLongitudeRef=E -GPSLongitude=${GPSlo/E/} -GPSLatitudeRef=N -GPSLatitude=${GPS/N*/} $GPSal"
}

# the associatitive arrays for dayly overview for smog and excess of EU levels
# per measurement station eg HadM NL10131 NETT
declare -A Info		# Info[Name|Location,DBtableName]

# amount of days to count
DAYS=${DAYS:-365}
YEAR=365
if [ "$DAYS" -gt "$YEAR" ] ; then YEAR="$DAYS" ; fi
# EU norm of year 
PHASE=2010	# EU 2008 second phase

# norms taken from Compendium
# http://www.compendiumvoordeleefomgeving.nl/indicatoren/nl0237-Nationale-luchtkwaliteit%3A-overzicht-normen.html?i=14-65

# associative array of pollutants and norms
declare -A POL
declare -A NORM
# for PM pollution phase 2 norms are used here
POL[pm_10]=24			# average of 24 hours per day
NORM[2010,pm_10,AVGyr]=20	# max PM10 avg for year avg phase 2 EU2008 norm
NORM[2010,pm_10,24]=50		# max PM10 day avg for count of days
NORM[2010,pm_10,24,max]=7	# max cnt days exceeding day avg phase 2 EU2008

POL[pm_25]=24			# average of 24 hours measurements per day
#NORM[2010,pm_25,AVGyr]=13	# max PM2.5 avg per year EPA 2009
#NORM[2010,pm_25,AVGyr]=25	# max PM2.5 avg per year, no norm yet
#NORM[2010,pm_25,AVGyr]=${NORM[2010,pm_10,AVGyr]} # turned off
NORM[2010,pm_25,AVGyr]=15	# max PM2.5 avg per year EPA 2006
NORM[2010,pm_25,24]=23		# max PM2.5 day avg for count of days
#NORM[2010,pm_25,24]=${NORM[2010,pm_10,24]}	# max PM10 no norm set yet
NORM[2010,pm_25,24]=35		# max PM2.5 day avg USA and CA norms
NORM[2010,pm_25,24,max]=1000	# max cnt days exceeding day average phase 2 EU2008 DISABLED

POL[so2]=24			# average of 24 hourly measurements per day
NORM[2010,so2,AVGyr]=1000	# disabled
NORM[2010,so2,24]=125
NORM[2010,so2,24,max]=3
NORM[2010,so2,1]=350
NORM[2010,so2,1,max]=3

POL[no2]=1			# max of 1 hour measurement
NORM[2010,no2,AVGyr]=40
NORM[2010,no2,1]=200
NORM[2010,no2,1,max]=18

POL[no]=24			# max of 24 hourly measurements per day
NORM[2010,no,AVGyr]=30
NORM[2010,no,24]=1000		# disabled
NORM[2010,no,24,max]=366	# disabled

POL[co]=8
NORM[2010,co,AVGyr]=1000000  	# DISABLED
NORM[2010,co,8]=10000
NORM[2010,co,8,max]=0

POL[o3]=8			# average of 8 hourly measurements
NORM[2010,o3,AVGyr]=1000	# disabled
NORM[2010,o3,8]=120
NORM[2010,o3,8,max]=25

POLdata=(SMOG SM_POL)			# pollutant names for day year averages
for P in ${!POL[*]}
do
    P=${P/_/} ; P=${P^^}
    # pollutant _ avg=year average  days#days > max day average
    # today=today average, last VAVG (4) days increasing (pos) of decreasing (neg)
    POLdata+=(${P}_avg ${P}_days ${P}_lastday ${P}_today ${P}_diff)
done
declare -A POLs			# pollutant values per station Pols[POL_type,station]
VAVG=4			# calculate sliding average over VAVG-1 days length VAVG+1 days

# Regeling van de Staatssecretaris van Infrastructuur en Milieu van 8 april 2016,
# nr. IENM/BSK-2016/75666, houdende wijziging van de Smogregeling 2010
# wijziging nationale informatiedrempel en alarmdrempel voor PM10
# Stof  informatiedrempel       	alarmdrempel	
# O3	180 µg/m3 (uurgemiddeld)	240 µg/m3 (uurgemiddelde)	
# SO2	350 µg/m3 (uurgemiddelde)	500 µg/m3 
# NO2	200 µg/m3 (uurgemiddelde)	400 µg/m3
# PM10	70 µg/m3 (daggemiddelde)	100 µg/m3 (daggemiddelde)	
# see definition SMOG: default here is 65 µg/m3 for only PM10 (only measured pol localy)
SMOG=${SMOG:-65} 	# smog PM10 >= 70, warn if PM10 is incr and above this level
# send also email to some addresses
SMOG_EMAIL="smog-alert@behouddeparel.nl"	# email alias address to send SMOG alert
FIJN_EMAIL="fijnstof@behouddeparel.nl"		# email alias address to send alert

# the CVS file with day averages exceeding day levels
#DAYS_YR_FILE=TB_EXTRA=$(mktemp --suffix=.YR_AVG)
if [ -z "$DEBUG" ]
then
    trap 'rm -f /tmp/tmp.*.YR_AVG /tmp/tmp.*.YR_AVG.HTML; exit' EXIT
fi

# have a nice date stamp in dutch style
YESTERDAY=$(date --date=yesterday "+%a %-d %b %Y")

# post-it website script details
POSTIT=@postit.svg.png
POSTIT_DIR=~teus/Pictures/
POSTIT_CMD=${POSTIT_DIR}postit.sh	# shell script to generate post-it image
if [ ! -x $POSTIT_CMD ]
then
    echo "Unable to run shell postit.sh to generate postit via $POSTIT_CMD" >&2
    exit 1
fi
# post-it default image name
IMG=@fijnstof.png
DPI=160         # dpi of generated png file
# svg generated file causes font family and (re)sizes problems on some OS's!


# location of website images
# post-it link to Horst aan de Maas measurents article
LINKPAGE=3904
# main measurements station
MAIN=HadM

# website location path
if [ -z "$WWW" ]
then
    for SITE in behouddeparel BdP
    do
        if [ -f /etc/apache2/sites-available/$SITE.conf ] ; then break ; fi
    done
    WWW=$(grep "DOCroot.*webdata" /etc/apache2/sites-available/$SITE.conf | head -1 | sed -e 's/.*DOCroot  *//' -e 's/\/ *$//' -e 's@/$@@')
    for DIR in sites/behouddeparel.nl ../cmsdata/BdP  cmsdata sites/default NULL
    do
        if [ -z "$WWW" ] ; then break ; fi
        if [ $DIR = NULL ] ; then WWW='' ; break ; fi
        DIR=$DIR/files/images
        if [ -d ${WWW}/$DIR ] ; then WWW=${WWW}/$DIR ; break ; fi
    done
fi
if [ -z "$WWW" ] || [ ! -d ${WWW} ] || [ ! -w ${WWW} ]
then
    echo "ERROR: Cannot find WWW dir $WWW." 1>&2
    exit 1
fi
# website owner
GRP=$(ls -ld "$WWW" | awk '{ print $4; }')

# use date in full NL style
function DATE() {
   local ARG1="$1"
   local ARG2="$2"
   shift ; shift
   if [ -n "$ARG1" ] && [ -z "${ARG1/--date=*/}" ]
   then
	ARG1=$(perl -e "\$_ = '$ARG1';" -e 's/\s*(ma|di|wo|do|vr|za|zo)\s+//; s/mrt/mar/; s/mei/may/; s/okt/oct/; print $_;')
   fi
   if [ -z "$ARG1" ] ; then date
   elif [ -z "$ARG2" ] ; then date "$ARG1" 
   else date "$ARG1" "$ARG2"  $*
   fi
}

# get index: args: [all|index|color|aqi|gom] pol1 value1 ...
# index is calculated from  max of a set of pollutants
# AQI standard: http://airnow.gov/index.cfm?action=aqibasics.aqi 
# AQHI standard: http://airqualityontario.com/science/aqhi_description.php
# LKI "standard proposal": IVM report 2014-0050
# http://www.rivm.nl/Documenten_en_publicaties/Wetenschappelijk/Rapporten/2015/mei/
# CAQI standard: EU CiteAir II, 2008 updated 2012, Common Information to European Air
# Ranges: AQI 1 - 500, AQHI 1 - 10, LKI 1 - 10, CAQI 1 - 100
# AQHI index is based on 3 pollutant indicators (O3, NO2, PM2.5)
# routine provides index, aqi value, color, gom or Google-o-meter image URL
# routine converts from ug/m3 (micro gram) to ppm/ppb (parts per million/billion)
# and other way around. Routine can accept temp eg 15.0C bar eg 10034.0mB (deflts)
# or give ppb as argument for ppb value indications.
# output selector:
# aqi (value), color, qual (message), index (class),
# gom (Google gauge), all (dflt), none
function INDEX()
{  
    local MYCMD=AQI ARG
    case "$1" in
    AQI)
                                # day average base
	MYCMD=maxAQI ; shift    # Air Quality Index algorithm (1968/2012, US, China ao)
    ;;
    AQHI)                       # Air Quality Health Index (2002/2013, Canada)
                                # day average base
	MYCMD=AQHI ; shift      # maxAQHI and AQHI are same routine
    ;;
    LKI)                        # LuchtKwaliteitsIndex algorithm (2015, RIVM Nld)
                                # hourly base !! need min-max per day as well.
        MYCMD=maxLKI ; shift
    ;;
    CAQI)                       # Common Air Quality Index (EU 2006/2012)
        MYCMD=maxCAQI ; shift
        # here used with 24h average measurements
        ARG="${*//pm_10/pm_10h24}" ; ARG="${ARG//pm_25/pm_25h24}" # day averages
    ;;
    esac
    if [ -z "$ARG" ] ; then ARG="$*" ; fi
    if [ ! -f ${INDEXSCRIPT} ] ; then INDEXSCRIPT=./AQI.pl ; fi
    if [ ! -f "${INDEXSCRIPT}" ]
    then
       echo "ERROR air quality index INDEX routine: AQI.pl script not found" 1>&2
       return 1
    fi
    perl -e "require '${INDEXSCRIPT}';" -e "${MYCMD} ('$ARG');"
    return $?
}

# INDEX AQI PM_10 7 PM_25 6
# INDEX AQI all no2 17 o3 48 PM_25 7
# INDEX AQHI gom no2 17 o3 48 PM25 7
# INDEX AQHI qual no2 17 o3 48 PM25 7
# INDEX AQHI index no2 17 o3 48 PM25 7
# INDEX LKI {all|gom|qual|index} pm_25 12 pm_10 14 no2 16 o3 15 (maybe roet 14?)

# get data for pollutants eg PM10 and PM2.5, limit exceesses with bang added
# for last 365 days, dayly/year average
# arg1: table name of measurement station
# optional arg2: use dayly average table
# prefer to use the DayAVG table filled with avg values from website
# DayAVG differ from avg calculations via mysql on original table.
# next is temp file data defintions for html update script $METINGEN
TEMPFILE=
function Get_KENMERKEN() {
    local TBL=$1 STATION=$1
    local KENMERKEN DAYCNT=8	# minimal DAYCNT hourly measurments for day average
    # do we have data for this station?
    if ! $MYSQL -e "SHOW TABLES;" | grep -q "^$TBL\$"
    then
        echo "Database does not has $TBL table." 1>&2
	return 1
    fi
    # if available use day average DB table
    if $MYSQL -e 'SHOW TABLES;' | grep -q "^${TBL}_DayAVG\$" \
	&& $MYSQL -e "
	    SELECT COUNT(*) FROM ${TBL}_DayAVG
	    WHERE DATEDIFF(datum,date_sub(now(), interval 1 day)) > -1" | grep -q 1
    then
	# table with day average is present use that one
	# this way we do not check on enough measurements in a day
	TBL+="_DayAVG" ; DAYCNT=0
    fi

    # get info of the station
    Info[Name,$STATION]=$($MYSQL -e "
	SELECT CONCAT('meetstation ',name,' (',stations.table,') van organisatie ',
	organisation) from stations where stations.table = '$1'")
    Info[Location,$STATION]=$($MYSQL -e "
	SELECT name from stations where stations.table = '$1'")
    Info[ID,$STATION]=$($MYSQL -e "
        SELECT id from stations where stations.table = '$1'")

    local P AA
    if [ -z "$TEMPFILE" ]
    then	# initialize TEMPFILE with global definitions, used by METINGEN
	TEMPFILE=$(mktemp --suffix=.YR_AVG)
	# next defines some settings for report generation and control
	echo "9999-12-31;station:${!POL[*]}" >${TEMPFILE}
	echo "date:YESTERDAY=$YESTERDAY" >>${TEMPFILE}  # day of avg values
	echo "date:TODAY=$(date "+%a %-d %b %Y")" >>${TEMPFILE}
	for P in ${!POL[*]}
	do
	  AA=${P/_/}
	  echo "norm:${AA^^}_MAXyr=${NORM[$PHASE,$P,AVGyr]}" >>$TEMPFILE
	  echo "norm:${AA^^}_MAX=${NORM[$PHASE,$P,${POL[$P]}]}" >>$TEMPFILE
	  echo "norm:${AA^^}_MAXdays=${NORM[$PHASE,$P,${POL[$P]},max]}" >>$TEMPFILE
	done
    fi
    echo -e "Name:$1=${Info[Name,$1]}\nLocation:$1=${Info[Location,$1]}" >>$TEMPFILE
    echo -e "Table:$1=$TBL" >>$TEMPFILE
    echo -e "Identity:$1=${Info[ID,$1]}" >>$TEMPFILE

    # collect names of sensors available for this station
    local SQL=() COND=() DEF=() SENSES=()
    for P in ${!POL[@]}
    do
	CNT=$($MYSQL -e "
            SELECT COUNT(*) FROM INFORMATION_SCHEMA.COLUMNS
            WHERE TABLE_NAME = '$TBL' AND COLUMN_NAME = '$P'")
	if [ -z "$CNT" ]
	then
	    echo "ERROR in MySQL. Exiting" >/dev/stderr
	    exit 1
        fi
	if [ "$CNT" -gt 0 ]
	then
	    SENSES+=($P)
	    if [ ${#SQL[@]} -gt 0 ] ; then SQL+=(","); fi
	    # request: per day the sensor value average
    	    SQL+=("'$P' AS 'pollutant', count($P) AS 'count', avg($P) AS 'day average'")
	    if [ ${#COND[@]} -gt 0 ] ; then COND+=("OR"); fi
    	    COND+=("${P}_valid")	# sensor values must be valid
	    # get the norms for max year average, max day average and max days 
	    DEF+=("SELECT 'DEF $P ${NORM[$PHASE,${P},AVGyr]} ${NORM[$PHASE,${P},${POL[${P}]}]} ${NORM[$PHASE,${P},${POL[${P}]},max]}' ;")
	fi
    done
    echo "9999-12-31;$1:${SENSES[@]}" >>$TEMPFILE
    # TO DO: we take here average per day also need if norm says 8 or 1 hour average
    # AQI uses one hour index, we do AQI on full day average
    KENMERKEN=$($MYSQL -e "
	SELECT '#DEF pollutant MAXavgyear MAXavgday MAXcntdays pollutant ...' ;
        ${DEF[*]}
	SELECT '#year month day pollutant hourcount dayaverage pollutant ...' ;
        SELECT
           YEAR(datum) AS jaar, MONTH(datum) AS maand, DAY(datum) AS dag,
           ${SQL[*]}
           FROM $TBL WHERE ( ${COND[*]} )
	   AND datediff(datum, now()) >= -$DAYS
           GROUP BY jaar, maand, dag
           ORDER BY jaar DESC, maand DESC, dag DESC ;
        " | sed 's/NULL/0/g' | \
      perl \
	    -e "\$FILE='$TEMPFILE';" \
	    -e "\$DAYS=${DAYS:-365}; \$YEAR=${YEAR:-365};" \
	    -e "\$TBL='${STATION:-HadM}'; \$VAVG=${VAVG:-4};" \
	    -e "%SMOG = ( PM10 => ${SMOG:-65}, O3 => 180, NO2 => 200, SO => 350, );" \
            -e "\$DAYCNT=${DAYCNT:-8};" \
	    -e "\$DEBUG=${DEBUG:-0};" \
	-e '
	    sub diff {
		my( $a, $b ) = @_;
	        if ( $a < $b ) { return ( 0 - int(abs($a-$b)+0.5)); }
	        return int(abs($a-$b)+0.5);
	    }
	    my %NORM; my %POL; my $file; my $fnd_smog = 0;

	    while (<STDIN>) {
	      if( /^\s*#/ ) { next; }
	      elsif( /^DEF/ ) {
		my @N = split /\s/, $_; next if $#N != 4;
		$NORM{$N[1]}{MAXyr}   = $N[2];
		$NORM{$N[1]}{MAX}     = $N[3];
		$NORM{$N[1]}{MAXdays} = $N[4];
		$POL{$N[1]}{CNT} = 0; $POL{$N[1]}{SUM} = 0; $POL{$N[1]}{DAYS}=0;
		for( my $i = 0; $i <= $VAVG; $i++ ) { $POL{$N[1]}{BUF}[$i] = 0; }
	      }
	      elsif( /20[0-9][0-9]/ ) {
		# sort order is last day of measuremets first
		my $dayexcess=0;
	       # only sliding average if more as 8 hours measurements
		my @vals = split /\s/, $_;
		# PM10 measurements count in field 4 day avg in field 5
		for( my $i = 3; $i <= $#vals; $i += 3) { # first 3 fields is date
		    next if not defined $vals[$i+1] or not defined $vals[$i+2];
		    # fields tuple: name, hour cnt, day average
		    # DAYCNT is minimal (hourly) measurements per day
		    if( ($vals[$i+1] > $DAYCNT) and ($vals[$i+2] ne "NULL" ) ) {
			if( not defined $NORM{$vals[$i]}{MAX} ) {
			    print STDERR "Dayly average MAX for $vals[$i] not defined!\n";
			} 
			elsif ( int($vals[$i+2]+0.5) > $NORM{$vals[$i]}{MAX} ) {
			    $POL{$vals[$i]}{CNT}++; # day average > norm day max
			    $dayexcess++;
			}
			$POL{$vals[$i]}{DAYS}++; # cnt and sum for yearly average
			$POL{$vals[$i]}{SUM} += $vals[$i+2];
			if( $POL{$vals[$i]}{CNT} <= $VAVG+1 ) {
			    # keep last averages for last $VAVG days
			    for( my $j = $VAVG-1; $j >= 0; $j-- ) {
				$POL{$vals[$i]}{BUF}[$j+1] = $POL{$vals[$i]}{BUF}[$j];
			    }
			    $POL{$vals[$i]}{BUF}[0]= $vals[$i+2];
			    if ( not defined $POL{$vals[$i]}{today} and $vals[$i+2] > 0 ) {
				# date sort order is last day first
			        $POL{$vals[$i]}{today} = $vals[$i+2]; # last day average
			        $POL{$vals[$i]}{lastday} = "$vals[0]-$vals[1]-$vals[2]";
				$POL{$vals[$i]}{lastday} =~ s/-/-0/g;
				$POL{$vals[$i]}{lastday} =~ s/-0([0-9][0-9])/-$1/g;
			    }
			}
		    }
		}
		# if any pollutant day avg higher add to data intermediate file
		if ( $dayexcess ) {
		    if ( not defined $file ) {
			open $file, ">>$FILE" or print STDERR "Cannot open $FILE for appending\n";
		    }
		    printf $file ("%s-%02d-%02d;%s:",$vals[0],$vals[1],$vals[2],$TBL);
		    for( my $i = 3; $i <= $#vals; $i += 3) {
			printf $file ("%s=%d ",$vals[$i],$vals[$i+2]);
		    }
		    printf $file ("\n");
		}
	      }
	    }
	    if ( defined $file ) { close $file; }
	    foreach my $pol (sort keys %POL) {
		next if not $pol;
		if (  not defined $NORM{$pol}{MAXyr} ) {
		  print STDERR "WARNING: MAX average per year for $pol not defined!\n";
		  $NORM{$pol}{MAXyr} = 1000000; # disabled
	        }
		if (  not defined $NORM{$pol}{MAX} ) {
		  print STDERR "WARNING: MAX average per day for $pol not defined!\n";
		  $NORM{$pol}{MAX} = 1000000; # disabled
	        }
		if (  not defined $NORM{$pol}{MAXdays} ) {
		  print STDERR "WARNING: MAX days per year for $pol not defined!\n";
		  $NORM{$pol}{MAXdays} = 1000000; # disabled
	        }

	 	my $nme = $pol; $nme =~ s/_//; $nme = uc($nme);

	        #  last day measurement with more as 8 measurements: XYZ_today
		printf("%s_today=%d%s\n", $nme, int($POL{$pol}{today}+0.5),
			(int($POL{$pol}{today}+0.5) >= $NORM{$pol}{MAX} ? "!" : "")
		    );
		printf("%s_lastday=%s\n", $nme, $POL{$pol}{lastday})
		    if defined $POL{$pol}{lastday} and $POL{$pol}{lastday};

	        # calculate the avg first part, second part and differential
	        for( my $i = 0; $i <= $VAVG-2; $i++) {
		    printf("BUF[%s,%d]=%3.1f\n",$nme,$i,$POL{$pol}{BUF}[$i]) if $DEBUG;
		    next if $i == 0;
		    $POL{$pol}{BUF}[0] += $POL{$pol}{BUF}[$i];
		};
	        $POL{$pol}{BUF}[0] = $POL{$pol}{BUF}[0]/($VAVG-1);
	        $POL{$pol}{BUF}[1] = 0;
		for( my $i = 2; $i <= $VAVG; $i++) {
		    printf("BUF[%s,%d]=%3.1f\n",$nme,$i,$POL{$pol}{BUF}[$i]) if $DEBUG and ($i > 2);
		    $POL{$pol}{BUF}[1] += $POL{$pol}{BUF}[$i];
		};
	        $POL{$pol}{BUF}[1] = $POL{$pol}{BUF}[1]/($VAVG-1);
	        printf("BUFavg[%s,0-%d]=%2.3f\nBUFavg[%s,2-%d]=%2.3f\n",
		    $nme, $VAVG-2, $POL{$pol}{BUF}[0], $nme, $VAVG, $POL{$pol}{BUF}[1])
		    if $DEBUG;
	        printf("%s_diff=%d\n", $nme, diff($POL{$pol}{BUF}[1],$POL{$pol}{BUF}[0]));
    
	        # SMOG alert for PM10 is more as level with increasing average
                if ( (defined $SMOG{$nme}) and (defined $POL{$pol}{today})
                    and ($POL{$pol}{today} >= $SMOG{$nme})
		    and ( diff($POL{$pol}{BUF}[1],$POL{$pol}{BUF}[0]) > 0) ) {
		    printf("SMOG=%d\n",$POL{$pol}{today}) if not $fnd_smog++;
		    printf("SM_POL=%s(%s)\n",$nme,$POL{$pol}{today});
	        }
    
	        # calculate yearly average
	        printf("%s_daycount=%d/%d\n", $nme, $POL{$pol}{DAYS}, $DAYS);
	        printf("%s_avg=%d%s\n", $nme,
		    int($POL{$pol}{SUM}/
			($POL{$pol}{DAYS} <= 0 ? 1 : $POL{$pol}{DAYS})
		    + 0.5) );
    
		# not extrapolated
	        printf("%s_DAYS=%d%s\n", $nme, $POL{$pol}{CNT},
                    ($POL{$pol}{CNT} >= $NORM{$pol}{MAXdays} ? "!" : "") );
	        # extrapolate # days exceeding limit max days
	        my $extra = ($POL{$pol}{CNT}*$DAYS/($POL{$pol}{DAYS} <= 0?
		 		1 : $POL{$pol}{DAYS}) );
	        $extra = int($extra*$YEAR/$DAYS +0.5);
                printf("%s_days=%d%s\n", $nme,
                    $extra, ($extra >= $NORM{$pol}{MAXdays} ? "!" : "") );
	    }
	    ')

    # extract from KENMERKEN per sensor
    # the year average (sliding year), last day average
    # days exceeding day average max, add air quality index values if available
    # push the data annotated with station/TBL name into the TEMPFILE
    local I A C U Q LD AQI VALUES=()
    for I in ${POLdata[@]}
    do
        OLD=${POLs[$I,$STATION]}
	POLs[$I,$STATION]=$(echo "$KENMERKEN" | grep "${I}=" | sed -e 's/.*=//' -e 's/^$/?/' | grep "[0-9]")
	if [ -z "${POLs[$I,$STATION]}" ] ; then POLs[$I,$STATION]=0 ; fi
	case "${I}XYZ" in
	*_todayXYZ)
            if ! perl -e "exit 1 if '${POLs[$I,$STATION]:-0}' -0.05 > 0" ; then
                for AQI in AQI LKI AQHI CAQI
                do
                  case "$AQI" in
                    AQI|LKI)    # get the aqi index for this pollutant for today
	            A=$(INDEX ${AQI} aqi "${I/_today/}" "${POLs[$I,$STATION]}")
                    if perl -e "exit 1 if '${A:-0}' -0.01 <= 0" ; then
	                echo "${I/_today/}_${AQI}:$STATION=${A}" >>$TEMPFILE
	                C=$(INDEX ${AQI} color "${I/_today/}" "${POLs[$I,$STATION]}")
	                echo "${I/_today/}_${AQI}color:$STATION=${C}" >>$TEMPFILE
	                U=$(INDEX ${AQI} gom "${I/_today/}" "${POLs[$I,$STATION]}")
	                echo "${I/_today/}_${AQI}gom:$STATION=${U}" >>$TEMPFILE
	                Q=$(INDEX ${AQI} qual "${I/_today/}" "${POLs[$I,$STATION]}")
	                echo "${I/_today/}_${AQI}qual:$STATION=${Q}" >>$TEMPFILE
                    fi
                  ;;
                  esac  
                  POLs[$AQI,$STATION]+=" ${I/_today/}"
                done
	    fi
	;;
	*_lastdayXYZ)
	    if [ -n "${POLs[$I,$STATION]/0/}" ] ; then  # lastday for pol defined
                if [ -z "$LD" ] ; then LD="${POLs[${I},$STATION]}" ; fi
                # keep track of last day encountered
                if [ $(DATE --date="${POLs[${I},$STATION]}" +%s) -gt $(DATE --date="$LD" +%s) ] ; then
                    LD="${POLs[${I},$STATION]}"
                fi
	        POLs[$I,$STATION]=$(DATE --date="${POLs[$I,$STATION]}" "+%a %-d %b %Y")
            else
                continue
	    fi
        ;;
        SM_POL*XYZ)
            if [ ! -z "${POLs[$I,$STATION]/0/}" ] ; then
                if [ -n "$OLD" ] ; then OLD="$OLD, " ; fi
                POLs[$I,$STATION]="$OLD${POLs[$I,$STATION]}"
            fi
            continue    # no need to add this to TEMPFILE
        ;;
	esac
	echo "$I:$STATION=${POLs[$I,$STATION]}" >>$TEMPFILE
    done
    if [ ! -z "${LD/0/}" ] ; then
        for  AQI in MaxAQI MaxLKI AQHI  CAQI
        do
            VALUES=()
            for A in ${POLs[${AQI/Max/},$STATION]}
            do
                if [ -n "${POLs[${A}_today,$STATION]}" ] ; then
                   VALUES+=("$A" "${POLs[${A}_today,$STATION]}")
                fi
            done
            if [ ${#VALUES[@]} -le 0 ] ; then continue ; fi
            # the aqi routine will filter
            A=$(INDEX ${AQI/Max/} aqi "${VALUES[@]}")
            if ! perl -e "exit 1 if ('${A:-0}' -0.05) < 0" ; then continue ; fi
            echo "${AQI}:$STATION=${A}"      >>$TEMPFILE
            for C in color gom qual ; do
                U=$(INDEX ${AQI/Max/} $C "${VALUES[@]//=/ }")
                echo "${AQI}$C:$STATION=${U}"      >>$TEMPFILE
            done
            if [ -n "${LD/0/}" ] ; then
                echo "${AQI}lastday:$STATION=${LD}" >>$TEMPFILE
            fi
            AQI_min_max ${AQI/M??/} $STATION ${LD} >>$TEMPFILE
        done
    fi

    if [ -n "$DEBUG" ]
    then
	echo "Temporary data file: $TEMPFILE" 1>&2
    fi
    return 0
}
# Get_KENMERKEN HadM
# Get_KENMERKEN NL10131

# get air quality min/max/avg of last day measurements of a station
# arg1: [LKI|AQHI|CAQI|AQI] (dflt HadM) optional
# arg2: station (dlft HadM) optional
# arg3: day YYYY-MM-DD (default last day in DB) optional
# returns print of MMAvg{AQI}:{station}={value} {pollutant value}...
function AQI_min_max() {
    local AQI=${1:-LKI} ; AQI=${AQI/M??/} # delete Max or Min
    if [ -n "$1" ] && echo "$1" | grep -q -E "^(AQI|AQHI|LKI|CAQI)"
    then
        shift
    fi
    local TBL=${1:-HadM}
    local THIS_DAY=${2:-$(date --date=yesterday "+%Y-%m-%d")}
    local LIMIT=4       # minimum of measurents in a day to be valid
    if  [ -z "${1/%201*/}" ] ; then THIS_DAY=$1 ; TBL=HadM ; fi
    declare -a DayHour=() POLs=()
    local -i I CNT=0 LAST_DAY FIRST_DAY
    local QRY=""  STRG INDX=0 MAX=0 MIN=999
    # get all possible relevant pollutants
    STRG=$(Get_pollutants "$TBL" | grep -P '(o3|pm_(10|25)|roet|no|co|so|nh)')
    POLs=($STRG)
    for(( I=0; I < ${#POLs[@]}; I++))
    do  
        if [ -n "$QRY" ] ; then QRY+=" or " ; fi
        QRY+="(not isnull(${POLs[$I]}) AND ${POLs[$I]}_valid)"
    done
    if [ -n "$THIS_DAY" ]
    then
        LAST_DAY=$($MYSQL -e "SELECT TO_DAYS('${THIS_DAY//\//-} 02:00:00')")
        if [ -z "$LAST_DAY" ] || [ "$LAST_DAY" = NULL ]
        then
            echo "ERROR ${AQI} index: date wrong format date $THIS_DAY" 1>&2
            return 1
        fi
        CNT=$($MYSQL -e "SELECT count(datum) FROM $TBL WHERE ($QRY) AND TO_DAYS(datum) = $LAST_DAY")
        # do we have enough measurements for this day?
        if (( $CNT <= $LIMIT )) ; then return 1 ; fi
    else
        LAST_DAY=$($MYSQL -e "SELECT TO_DAYS(datum) FROM $TBL ORDER BY datum DESC LIMIT 1")
        FIRST_DAY=$($MYSQL -e "SELECT TO_DAYS(datum) FROM $TBL LIMIT 1")
        for (( ; FIRST_DAY <= LAST_DAY ; LAST_DAY-- ))
        do
            CNT=$($MYSQL  -e "SELECT count(datum) FROM $TBL WHERE ($QRY) AND TO_DAYS(datum) = $LAST_DAY")
            if (( $CNT > $LIMIT )) ; then break ; fi
        done 
        if (( FIRST_DAY > LAST_DAY ))
        then
            echo "ERROR: ${AQI} index; cannot find a day with measurements." 1>&2
            return 1
        fi
    fi
    QRY=${POLs[*]}
    QRY=$(echo $QRY | sed -e 's/[^a-z0-9_]/ /g' -e 's/   */ /g' -e "s/\\([a-z0-9][a-z0-9_]*\\)/'\\1',\\1/g")
    QRY=$(\
        $MYSQL -e "SELECT ${QRY// /,} FROM $TBL WHERE TO_DAYS(datum) = $LAST_DAY" |\
        sed -e 's/NULL/0/g' -e 's/\t/ /g' -e 's/  */@/g'\
        )
    DayHour=($QRY)
    local MAXindex=-1 MINindex=-1 AVGindex AVG
    CNT=0
    for ((I=0; I < ${#DayHour[@]}; I++))
    do
        DayHour[$I]="${DayHour[$I]//@/ }"       # set spaces back again
        INDX=$(INDEX ${AQI} aqi ${DayHour[$I]})
        if ! perl -e "exit 1 if '${INDX:-0}' - 0.01 <= 0"
        then
            DayHour[$I]=""
            continue
        fi
        ((CNT++))
        if perl -e "exit 1 if '${INDX:-0}' - '${MAX:-0.05}' <= 0"
        then MAX=$INDX ; MAXindex=$I
        fi
        if perl -e "exit 1 if '${INDX:-0}' - '${MIN:-0}' >= 0"
        then MIN=$INDX ; MINindex=$I
        fi
    done

    QRY=${POLs[*]}
    QRY=$(echo $QRY | sed -e 's/[^a-z0-9_]/ /g' -e 's/   */ /g' -e "s/\\([a-z0-9][a-z0-9_]*\\)/'\\1',avg(\\1)/g")
    AVGindex=$(\
        $MYSQL -e "SELECT ${QRY// /,} FROM $TBL WHERE TO_DAYS(datum) = $LAST_DAY" |\
        sed -e 's/NULL/0/g' -e 's/\t/ /g' -e 's/  */ /g'\
        )
    AVG=$(INDEX ${AQI} aqi $AVGindex)
    if ! perl -e "exit 1 if '${AVG:-0}' - 0.05  >= 0"
    then
        echo "MMAvg${AQI}:$TBL=$AVG $AVGindex"
        echo "MMGom${AQI}:$TBL=$(INDEX ${AQI} gom $AVGindex)"
        echo "MMCol${AQI}:$TBL=$(INDEX ${AQI} color $AVGindex)"
        echo "MMQual${AQI}:$TBL=$(INDEX ${AQI} qual $AVGindex)"
        if (( $MINindex >= 0 )) ; then echo "MMMin${AQI}:$TBL=$MIN ${DayHour[$MINindex]}" ; fi
        if (( $MAXindex >= 0 )) ; then echo "MMMax${AQI}:$TBL=$MAX ${DayHour[$MAXindex]}" ; fi
    fi
    if [ -z "$THIS_DAY" ]
    then
        THIS_DAY=$($MYSQL -e "SELECT UNIX_TIMESTAMP(datum) FROM $TBL WHERE TO_DAYS(datum) = $LAST_DAY LIMIT 1")
        THIS_DAY=$(date --date=@$THIS_DAY "+%a %-d %b %Y")
    else
        echo "MMDay${AQI}:$TBL=$THIS_DAY"
    fi
}
# AQI_min_max AQHI NL10131
# AQI_min_max LKI NL10131
# AQI_min_max AQI NL10131
# AQI_min_max CAQI NL10131

# send smog and/or PM10 day email alert	
function Smog_ALERT() {
    # make smog noise on the website?
    if [ -z "${POLs[SMOG,$1]/0/}" ]  ; then return 0 ; fi
    if [ ! -z "${POLs[PM10_lastday]/0/}" ] || [ "${POLs[PM10_lastday,$1]}" != "${YESTERDAY}" ]
    then
	echo "Smog alert warning: skipped for date ${POLs[PM10_lastday,$1]/#0/\?}" 1>&2
	return 1
    fi
    # we might use the LKI index here if LKI > 4
    local LP PWD SP
    # post-it to the website, time to live one day
    if [ -n "${LINKPAGE}" ] ; then LP="NODE=$LINKPAGE" ; fi
    PWD=$(pwd)
    SP="${POLs[SM_POL,$1]//)/ µg\/m³)}"
    cd $POSTIT_DIR ;
    rm -f ${POSTIT/.png/}.png ${POSTIT/.svg.png/.png}
    if [ -x ${POSTIT_CMD} ] ; then
       ${POSTIT_CMD} BdP push $LP tomorrow "${Info[Location,$1]}" "${YESTERDAY}" "smog ALERT" "voor ${SP}." ;
       echo "Generated smog alert post-it on website BdP." 1>&2
    fi
    cd $PWD
    if [ -n "${SMOG_EMAIL}" ] && \
        mail -s "smog ALERT" ${SMOG_EMAIL} <<<"Dit is een automatisch bericht. Een smog alert: de volgende stof(fen) zijn momenteel boven hun µg/m³ niveau, zoals ${SP}, en de waarde neemt waarschijnlijk nog toe."
    then
        echo "Sent smog alert emails to $SMOG_EMAIL." 1>&2
    fi
    return $?
}

# report for all TBLS the state of pollution
function Report_States() {
    local Station
    for Station in ${TBLS}
    do
	if [ "$Station"  = "$1" ] ; then continue ; fi
	Report_State "$Station"
    done
    return $?
}

# report on stdout the pollution state for one station
function Report_State() {
    local pol
    if [ -z "$1" -o -z "${Info[Location,$1]}" ] ; then return 1 ; fi
    for I in ${POLdata[@]}
    do
	if [ "$I" = SMOG  ] || [ "${I/*_avg/avg}" = "avg" ] || [ "${I/*_lastday/lastday}" = "lastday" ]
	then
	    continue
	fi
	if [ -z "${POLs[$I,$1]/\!/}" ] || [ "${POLs[$I,$1]/\!/}" -le 1 ]
	then
	    POLs[$I,$1]='?'
	fi
    done

    echo "Het luchtkwaliteits-${Info[Name,$1]:-\?}:"
    echo "Het gemeten jaargemiddelde van enkele vervuilers zijn:"
    for pol in ${!POL[@]}
    do
	local P=${pol^^} ; P=$(echo $P | sed s/PM_/PM/)
	if [ $($MYSQL -e "
            SELECT COUNT(*) FROM INFORMATION_SCHEMA.COLUMNS
            WHERE TABLE_NAME = '$1' AND COLUMN_NAME = '$pol'") -le 0 ]
        then continue ; fi
        if [ -z "${POLs[${P}_avg,$1]}" ] ; then POLs[${P}_avg,$1]=0 ; fi
	echo -n "    ${P/PM25/PM2.5} luchtwaliteit:"
	if [ "${POLs[${P}_avg,$1]/\!/}" -le 0 ]
	then
	   echo " is de 365 laatste dagen niet meer gemeten."
	   continue
        fi
	echo
        if [ "${POLs[${P}_avg,$1]:-\?}" = '?' ]
        then
	    echo -e "\tgemiddelde over laatste 365 dagen is onbekend µg/m³"
        else
	    echo -e "\tgemiddelde over laatste 365 dagen: ${POLs[${P}_avg,$1]:-\?} µg/m³"
        fi
	if [ ${POL[$pol]} = 24 -a ${NORM[$PHASE,$pol,${POL[$pol]},max]} -lt 365 ]
	then
            echo -n -e "\taantal dagen van verschrijdingen van max ${NORM[$PHASE,$pol,${POL[$pol]}]} µg/m³ (max ${NORM[$PHASE,$pol,${POL[$pol]},max]} dagen)"
            if [ "${POLs[${P}_days,$1]:-\?}" = '?' ]
            then
                echo " is onbekend."
            else
	        echo ": ${POLs[${P}_days,$1]:-\?} dagen."
            fi
        fi
        echo -n -e "\tlaatste daggemiddelde gemeten is "
        if [ "${POLs[${P}_today,$1]:-\?}" = '?' ]
        then
	    echo "onbekend."
        else
            echo -n "${POLs[${P}_today,$1]/#0/\?} µg/m³"
	    echo -n " op ${POLs[${P}_lastday,$1]/#0/\?}"
	    if [ "${POLs[${P}_diff,$1]:-\?}" != '?' ]
	    then
	       echo -n " en stijgt/daalt met ${POLs[${P}_diff,$1]} µg/m³ per dag over $((VAVG+1)) dagen."
	    fi
	    echo
        fi
    done
    echo
    return 0
}

# send an email alert if PM values exceed a norm and report other station measurments
function Day_ALERT() {
    if echo "${POLs[PM25_today,$1]} ${POLs[PM10_today,$1]}" | grep -q '!'
    then
        if [ "${POLs[PM10_diff,$1]/\!/}" -ge 0 ] || [ "${POLs[PM25_diff,$1]/\!/}" -ge 0 ]
        then
            if [ -n "${FIJN_EMAIL}" ] 
            then
                (echo -e "\nDit is een automatisch bericht.
Als er een overschrijding is van de EU 2010 norm 2e fase wordt dat aangegeven met een uitroepteken.\n" ;
	    Report_State "$1" ;
		echo "Andere regionale meetstations:" ;
		Report_States "$1" ) \
                | mail -s "fijnstof nivo overschrijding in ${Info[Location,$1]}" ${FIJN_EMAIL}
            fi
        fi
    else
	return 1
    fi
    return 0
}

# arg1: table id for the station
# do a post-it of PM values of a station, and push it to the website
function Postit() {
    # generate the post-it
    local PWDIR=$(pwd)
    local MY_secs=$(date --date=yesterday +%s)
    local MY_date=$(date --date=yesterday "+%a %-d %b %Y")
    # post-it generation
    cd "$POSTIT_DIR"
    if [ "$?" != 0 -o ! -x ${POSTIT_CMD} ]
    then
        echo "ERROR: no dir $POSTIT_DIR or postit.sh script found." 1>&2
        return 1
    fi
    rm -f ${POSTIT/.png/}.png ${POSTIT/.svg.png/.png}
    # take the most up to date measurement day as day of post-it
    if [ ! -z "${POLs[PM10_lastday,$1]/0/}" ] && [ $(DATE --date="${POLs[PM10_lastday,$1]/#???/}" +%s) -lt $MY_secs ]
    then
	MY_date=$(DATE --date="${POLs[PM10_lastday,$1]/#???/}" +%s)
	MY_secs=$(DATE --date="${POLs[PM10_lastday,$1]/#???/}" +%s)
    fi
    if [ ! -z "${POLs[PM25_lastday,$1]/0/}" ] && [ $(DATE --date="${POLs[PM25_lastday,$1]/#???/}" +%s) -gt $MY_secs ]
    then
	MY_date=$(DATE --date="${POLs[PM25_lastday,$1]/#???/}" +%s)
	MY_secs=$(DATE --date="${POLs[PM25_lastday,$1]/#???/}" +%s)
    fi
    # maybe we have an AQI index defined for yesterday
    declare -a AQI
    local AQIstrg=LKI AQIday=@${MY_secs}  # we use LKI air qual index here
    AQI=($(AQI_min_max $AQIstrg HadM $(date --date=${AQIday} "+%Y-%m-%d") | sed -e 's/  */@/g'))
    if echo "${AQI[@]}" | grep -q "$AQIstrg:"
    then
        local -i I
        local MIN="0" MAX="0" AVG="0"
        for (( I=0; I < ${#AQI[@]} ; I++ ))
        do
            case "${AQI[$I]}" in
            MMMax*:*)
                MAX=${AQI[$I]/@*/} ; MAX=$(perl -e "print int('${MAX/*=/}'+0.5)")
            ;;
            MMMin*:*)
                MIN=${AQI[$I]/@*/} ; MIN=$(perl -e "print int('${MIN/*=/}'+0.5)")
            ;;
            MMAvg*:*)
                AVG=${AQI[$I]/@*/} ; AVG=${AVG/*=/}
            ;;
            MMDay*:*)
                AQIday=${AQI[$I]/@*/} ; AQIday=${AQIday/*=/}
            ;;
            esac
        done
        if [ "$MIN" -lt 4 ]
        then
            MIN="[gS($MIN)"
        else
            MIN="[rS($MIN)"
        fi 
        if [ "$MAX" -lt 4 ]
        then
            MAX="[gS($MAX)"
        else
            MAX="[rS($MAX)"
        fi 
        if perl -e "exit 1 if '${AVG:-0}' -4 >= 0;"
        then
            AQIstrg="${AQIstrg} avg [gS($AVG), min:${MIN} max:${MAX}"
        else
            AQIstrg="${AQIstrg} avg [rS($AVG), min:${MIN} max:${MAX}"
        fi 
    fi

    # styling is defined as [X(string) where X is one or more for
    # u super, d sub, s smaller, S larger,
    # M msam10 symbol font, trangles: J left, I right, N up, H down,
    # r red, g green, G gray.
    # patch the post-it with up/down arrows
    local PM10loc='[rMS(N)'	# up
    if [ "${POLs[PM10_diff,$1]}" = 0 ]
    then
        PM10loc='[SGM(J)'
    elif [ "${POLs[PM10_diff,$1]}" -lt 0 ]
    then
        PM10loc='[SgM(H)'
    fi
    local PM25loc='[SrM(N)'	# up
    if [ "${POLs[PM25_diff,$1]}" = 0 ]
    then
        PM25loc='[SGM(J)'
    elif [ "${POLs[PM25_diff,$1]}" -lt 0 ]
    then
        PM25loc='[SgM(H)'
    fi
    # the arguments have all types of size changes and changes to get arrows
    # use of s(x) smaller, d(x) sub script, see post-it help
    # todo: YESTERDAY make it equal to ${POLs[PM10_lastday,$1]}
    if [ -n "$AQIstrg" ]
    then
      DPI=${DPI:-90} ${POSTIT_CMD} "LuchtKwaliteitsIndex" \
        "${1/HadM/Horst a\/d Maas}" \
	"[s(op) $(DATE --date=${AQIday} '+%a %-d %b %Y')" \
	"$AQIstrg" \
	"PM[d(10):${POLs[PM10_today,$1]/\!/[rS(!)} µg/m[u(3)${PM10loc}, PM[d(2.5):${POLs[PM25_today,$1]/\!/[rS(!)} µg/m[u(3)${PM25loc}" \
	"${POLs[PM10_days,$1]/\!/[rS(!)} dagen >${NORM[$PHASE,pm_10,${POL[pm_10]},max]} µg/m[u(3)" \
	"jaar≈ PM[d(10):${POLs[PM10_avg,$1]/\!/[rS(!)} PM[d(2.5):${POLs[PM25_avg,$1]/\!/[rS(!)}" \

    else
      DPI=${DPI:-90} ${POSTIT_CMD} "fijnstof $1" \
	"$(DATE --date=@${MY_secs} '+%a %-d %b %Y')" \
	"PM[d(10):${POLs[PM10_today,$1]/\!/[rS(!)}${PM10loc}, PM[d(2.5):${POLs[PM25_today,$1]/\!/[rS(!)}${PM25loc}" \
	"${POLs[PM10_days,$1]/\!/[rS(!)} dagen >${NORM[$PHASE,pm_10,${POL[pm_10]},max]} µg/m[u(3)" \
	"jaar≈ PM[d(10):${POLs[PM10_avg,$1]/\!/[rS(!)} PM[d(2.5):${POLs[PM25_avg,$1]/\!/[rS(!)}" \

    fi

    if [ ! -f "$POSTIT" ]
    then
        echo "ERROR: cannot generate $POSTIT." 1>&2
        exit 1
    else
        mv "$POSTIT" "$IMG"
    fi
    if [ ! -f "${POSTIT/%.png/.svg}" ]
    then
        echo "WARNING: cannot generate ${POSTIT/%.png/.svg}" 1>&2
    else
        mv "${POSTIT/%.png/.svg}" "${IMG/%.png/.svg}"
    fi
    exiftool -q -m -overwrite_original_in_place  $(Get_GPS table $1) "$IMG"

    #################### finally to the website
    # push the post-it to the website
    if [ ! -d "$WWW" ]
    then
        echo "No WWW $WWW, post-it image is here: $(pwd)/$IMG"
        exit 0
    elif ! mv "$IMG" "$WWW/$IMG" \
        || ! mv "${IMG/%.png/.svg}" "$WWW/${IMG/%.png/.svg}"
    then
        echo "ERROR: cannot move $IMG or ${IMG/%.png/.svg} to $WWW." 1>&2
        exit 1
    fi
    chgrp $GRP "$WWW/$IMG" "$WWW/${IMG/%.png/.svg}"
    echo "New particular matter WWW post-it image for $1: $IMG and ${IMG/%.png/.svg}"
    return 0
}

# todays overview of several regional stations and last 365 days
# year avg exceeding info in table format for several stations
function Year_Overview() {
    # for now we only do HTML output in node3911.html
    local FILE=$1
    if [ ! -f "$1" ] ; then return 1; fi
    if [ ! -x "$METINGEN" ]
    then
	echo "ERROR $METINGEN script not found, cannot update website article." 1>&2
	return 1
    fi
    if [ -n "$VERBOSE" -a "$VERBOSE" = "/dev/stdout" ] ; then KEY="$KEY -d" ; fi
    if [ -n "$ALIAS" ]
    then
	NODE=$($WWWMYSQL -e "
	    SELECT source FROM url_alias WHERE alias = '$ALIAS'" | sed 's/^node.//')
    fi
    local HTMLTMP=$(mktemp --suffix=.YR_AVG.html)
    local PWD=$(pwd)

    cd $(dirname "$METINGEN");  # create new html dagwaarden.html file
    if [ -z "$DBUSER}" ] ; then export DBUSER=$(GET_passwd USER) ; fi
    if [ -z "$DBPASS}" ] ; then export DBPASS=$(GET_passwd) ; fi
    if [ -z "$WWWDBPASS" ] || [ -z "$WWWDBUSER" ]
    then
        echo "WARNING: using DB credentials for WWW DB access!" 1>&2
    fi
    if [ -z "$WWWDBPASS" ] ; then export WWWDBPASS=$DBPASS ; fi
    if [ -z "$WWWDBUSER" ] ; then export WWWDBUSER=$DBUSER ; fi
    if [ -n "$DEBUG" ]
    then
        echo "CALL to: DBUSER=$DBUSER DBPASS=$DBPASS WWWDBUSER=$WWWDBUSER WWWDBUSER=$WWWDBUSER ${METINGEN} ${FORECAST} $KEY '$TEMPFILE' '${NODE:-3911}' '${HTMLTMP/.html/}' " 1>&2
    fi
    ${METINGEN} $KEY "$TEMPFILE" "${NODE:-3911}" "${HTMLTMP/.html/}" 2>&1 | \
	    grep -v -e "New HTML file" -e "Get template data" 1>&2

    if [ ! -s "${HTMLTMP}" ]
    then
        echo "Day_results.sh $METINGEN failed to generate $HTMLTMP from $TEMPFILE" 1>&2
        echo "Failing command: DBUSER=$DBUSER DBPASS=$DBPASS WWWDBUSER=$WWWDBUSER WWWDBUSER=$WWWDBUSER ${METINGEN} $KEY '$TEMPFILE' '${NODE:-3911}' '${HTMLTMP/.html/}'" 1>&2
        cp $TEMPFILE /var/tmp/@KENMERKEN.data
        echo "Saved data file in /var/tmp/@KENMERKEN.data" 1>&2
        cd "$PWD"
        return 1
    fi
    Push_HTML "$HTMLTMP" "${NODE:-3911}"
    cd "$PWD"
    return 0
}

# next will read the HTML file from the website in Drupal
# dagwaarden.html is put in ...files/images/../luchtmetingen
# to be found in file_managed DB table
# TO DO: move private to public dir on website!
#<?php
#    $uri = "private://luchtmetingen/dagwaarden.html";
#    $fhandle = fopen($uri,'r');
#    do {
#        //$result = fread($fhandle,filesize($uri));
#        $result = fread($fhandle,1024);
#        echo $result;
#    } while ( $result );
#    fclose($fhandle);
#?>

function Push_HTML() {
    local FILE=$1
    local NAME=dagwaarden.html
    if [ ! -f "$FILE" ]
    then
	echo "ERROR: Day_results/Push_HTML; File $FILE not existant!" 1>&2
        return 1
    fi
    local FID=$($WWWMYSQL -e "SELECT uri FROM file_managed WHERE filename = '$NAME' ")
    if [ -z "$FID" ]
    then
        if ! $WWWMYSQL -e "
            INSERT INTO file_managed
	        (uid,filename,uri,filemime,filesize,status,timestamp)
	        values
	        (1,'$NAME','private://luchtmetingen/$NAME',
	           'text/html',18764,1,1432553532)"
        then
	    echo "ERROR: failed to create first instance in file managed." 1>&2
	    return 1
        fi
        FID=$($WWWMYSQL -e "
	    SELECT uri FROM file_managed WHERE filename = '$NAME' ")
    fi
    # public WWW (/webdata/Drupal/sites/sites/behouddeparel.nl/files/images/../luchtmetingen/dagwaarden.html) -> 
    #           /webdata/Drupal/cmsdata/BdP/files/images/../luchtmetingen/dagwaarden.html
    # sites/sites/behouddeparel.nl/files -> cmsdata/BdP/files
    if [ -z "${FID/private:\/\/*/}" ]
    then
        FID=${FID/private:\/\//${WWW}\/..\/}
        FID=${FID/sites\/sites\/behouddeparel.nl/cmsdata\/BdP}
    elif [ -z "${FID/public:\/\/*/}" ]
    then
        FID=${FID/public:\/\//${WWW}\/..\/}
    fi
    if !  cp -f "$FILE" "$FID"
    then
        echo "ERROR: Cannot copy $FILE to dir $FID." 1>&2
        return 1
    elif [ -f "$FID" ]
    then
        chgrp $GRP "$FID"
        local SIZE=$(perl -e "\$size = -s \"$FID\"; print \"\$size\";")
        local FDATE=$(perl -e "\$size = (stat \"$FID\")[9]; print \"\$size\";")
	local NID=$($WWWMYSQL -e "
	        SELECT source FROM url_alias WHERE alias = 'fijnstof-metingen'")
	NID=${NID/node\//}
        if ! $WWWMYSQL -e "
	    UPDATE file_managed SET timestamp = $FDATE, filesize = $SIZE WHERE filename = '$NAME' " || \
	   ! $WWWMYSQL -e "
	    UPDATE node SET changed = $FDATE WHERE nid = ${NID:-3914}"
	then
	    echo "WARNING: failed to adjust $NAME in managed_file table." 1>&2
        fi
    else
        return 1
    fi
    echo "Pushed new $NAME file to website and updated $WWWDB DB file table." 1>&2
    return 0
}

#Year_Overview /tmp/tmp.BBVleqsXam.YR_AVG
function Full_Overview() {
    echo -e "Overzicht van regionale fijnstof meetstations van $YESTERDAY\n"
    # collect KENMERKEN for all stations
    for Stat in $*
    do
	if Get_KENMERKEN "$Stat"
	then
	    Report_State "$Stat"
	fi
    done
    Year_Overview "$TEMPFILE" $*
}

# collect PM10 PM2.5 movies of the Netherlands
# the movies shows levels from yesterday up to next few days
# the movie originated from project by RIVM, KNMI and TNO
# http://projects.knmi.nl/lukwa/lkv/YYYY/MM/DD/{ozone,p10,p25}_movie.gif
function Collect_Movie() {
   declare -i Day
   local T IMG RTS=0
   if [ ! -d "$WWW" ]
   then
	echo "ERROR: Do not have webdir image target directory!\n" 1>&2
	return 1
   fi
   # try to avoid blacklisting of research process
   if [ -n `pidof privoxy` ]
   then
       export HTTP_PROXY=http://127.0.0.1:8118/
       export http_proxy=http://127.0.0.1:8118/
   else
       echo "WARNING: Should have privoxy service running.." 1>&2
   fi
   for T in $*
   do
     T=${T/PM/p}
     case "$T" in
        ozone|p10|p25)
        IMG=${T}_movie.gif
     ;;
     *)
	echo "WARNING: unknown type $T!" 1>&2
	RTS=1
	continue
     ;;
     esac
     rm -f /var/tmp/$IMG
     # we try up to 3 days ago to get the movie
     for(( Day=1; Day <= 3; Day++))
     do
        # seems http proxy results only in one picture of the movie serie
        if wget -q -O /var/tmp/$IMG --no-proxy \
	  http://projects.knmi.nl/lukwa/lkv/$(date --date="$Day day ago" +"%Y/%m/%d")/$IMG \
	  && [ -f /var/tmp/$IMG ] \
	  && file /var/tmp/$IMG | grep -q image
	then
	    break
	else
	   rm -f /var/tmp/$IMG
	fi
     done
     if [ -f /var/tmp/$IMG ]
     then
	cp /var/tmp/$IMG $WWW/
	chgrp www-data $WWW/$IMG
	echo "Created emission movie $WWW/$IMG." 1>&2
     else
        echo "WARNING: Could not grab $IMG from projects.knmi.nl website!" 1>&2
	RTS=1
     fi
   done
   return $RTS
}

if [ -z "$1" ]
then
    Get_KENMERKEN "${MAIN:-HadM}"
    if [ -z "${POLs[PM10_avg,${MAIN:-HadM}]}" ]
    then
	echo "ERROR: failed to collect day results for particular matter PM10." 1>&2
	exit 1
    fi
    Postit "${MAIN:-HadM}"
    Smog_ALERT "${MAIN:-HadM}"
    if ! echo "${POLs[PM25_today,${MAIN:-HadM}]} ${POLs[PM10_today,${MAIN:-HadM}]}" | grep -q '!'
    then
        exit 0
    fi
    # collect KENMERKEN for all stations
    for Station in ${TBLS}
    do
        if [ "$Station" = "${MAIN:-HadM}" ] ; then continue ; fi
        # get status regional stations as well
        Get_KENMERKEN "${Station}"
    done
    Day_ALERT "${MAIN:-HadM}"
    # and push recent movies of pollutants PM10 and PM2.5 in Nld to website
    Collect_Movie p10 p25
else
    case "$1" in
    KENMERKEN|kenmerken)
	shift
	Get_KENMERKEN "${MAIN:-HadM}"
	for Station in $*
        do
	   if [ "$Station" = "${MAIN:-HadM}" ] ; then continue ; fi
	   Get_KENMERKEN "${Station}"
	done
	if [ -f "$TEMPFILE" ]
	then
	    if [ -n "${DEBUG}" ]
	    then
		echo "-------------- Pols variables" >>$TEMPFILE
		for Station in ${!POLs[@]}
		do
		   echo "POLs[$Station]=${POLs[$Station]}"
		done | sort -t, -k2 -k1 >>$TEMPFILE
	    fi
	    cp "$TEMPFILE" KENMERKEN.data
	    echo "KENMERKEN data of $* to be collected in KENMERKEN.data" 1>&2
	else
	    echo "ERROR no $TEMPFILE for kenmerken $*" 1>&2
	fi
    ;;
    [Mm]ovies|[cC]ollect)
	shift
	Collect_Movie ${*:-p10 p25}
    ;;
    REPORT|[rR]eport|[oO]verview|OVERVIEW)
        shift
	for Station in $*
	do
	    if Get_KENMERKEN "$Station"
            then
                Report_State "$Station"
            fi
	done
    ;;
    *)
        Full_Overview $* >$VERBOSE
    ;;
    esac
fi
exit $?
