#!/usr/bin/perl -w
# year avg exceeding info in table format for several stations
# arg1: data file
# optional arg2: either template file or Drupal node number
# optional arg3: push data into database either via json or qry
# if arg3 is number node nr is used, else created as a file in SQL or JSON format
# output is on STDOUT

# $Id: metingen.pl,v 3.39 2016/12/12 16:59:17 teus Exp teus $
# Copyright (C) 2015, Teus Hagen, the Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

    my $scan_only = '^(pm_10|pm_25)$' ; # scan only html file for these DB pollutants
    # add forecast weather and PM2.5 table
    my $forecast = 'FORECASTING';   # dflt: not, otherwise use string as id e.g. FORECASTING
    # AQI, MaxAQI and AQHI table printout only if option is defined (=1)
    # and index values are provided (=2).
    my %AQIprint = (
        aqi     => 0, 	# AQI index HTML printout ?
        maxaqi  => 0,   # MAX AQI index HTML printout ? (depricated)
        caqi    => 0,   # Common AQI Europe printout ?
        lki     => 0,  	# LKI index HTML printout ?
        maxlki  => 0,  	# MAX LKI index HTML printout ? (depricated)
        aqhi    => 0, 	# MAX AQHI index HTML printout ?
        mmaqi   => 0,  	# min/max AQI index HTML printout ?
        mmlki   => 0, 	# min/max LKI index HTML printout ?
     );
    # table of how in definitions they are identified, max is max of index value
    my %AQItable = (
        caqi => {
            pri   => 6,
            colorAQ => 'caqicolor', maxAQ => 'caqiqual',
            maxLD => 'caqilastday', maxGOM => 'caqigom',
            using => ['NO2', 'PM10','PM2.5', 'CO', 'SO2', 'O3',],
            name  => 'EU Common Air Quality Index',
            range => 100,
        },
        aqhi => {
            pri   => 5,
            colorAQ => 'aqhicolor', maxAQ => 'aqhiqual',
            maxLD => 'aqhilastday', maxGOM => 'aqhigom',
            using => ['NO2', 'O3', 'PM2.5', ],
            name  => 'Air Quality Health Index',
            range   => 10,
        },
        lki => {
            pri   => 1,
            colorAQ => 'lkicolor', maxAQ => 'lkiqual',
            maxLD => 'lkilastday', maxGOM => 'lkigom',
            using => ['NO2', 'O3', 'PM10','PM2.5', '(roet)',],
            name  => 'LuchtKwaliteitsIndex',
            range => 10,
        },
        maxlki => {
            pri   => 2,
            colorAQ => 'maxlkicolor', maxAQ => 'maxlkiqual',
            maxLD => 'maxlkilastday', maxGOM => 'maxlkigom',
            using => ['NO2', 'O3', 'PM10','PM2.5', '(roet)',],
            name  => 'max of LuchtKwaliteitsIndex',
            range => 10,
        },
        aqi => {
            pri   => 3,
            colorAQ => 'aqicolor',  maxAQ => 'aqiqual',
            maxLD => 'aqilastday', maxGOM => 'aqigom',
            using => ['NO2', 'O3', 'PM10','PM2.5', 'SO2', 'CO',],
            name  => 'Air Quality Index', # per pollutant
            range => 500,
        },
        maxaqi => {
            pri   => 4,
            colorAQ => 'maxaqicolor',  maxAQ => 'maxaqiqual',
            maxLD => 'maxaqilastday', maxGOM => 'maxaqigom',
            using => ['NO2', 'O3', 'PM10','PM2.5', 'SO2', 'CO',],
            name  => 'max of Air Quality Indexes',
            range => 500,
        },
    );    

    my %DB;
	no warnings "experimental";

	# take DB credentials either defaults or from environment
        $DB{DATA}{DB} = $ENV{"DB"} if $ENV{"DB"}; 
	$DB{DATA}{DB} = "luchtmetingen" if not $DB{DATA}{DB};
        $DB{WWW}{DB} = $ENV{"WWWDB"} if $ENV{"WWWDB"}; 
	$DB{WWW}{DB} = "parel7" if not $DB{WWW}{DB};
        $DB{DATA}{HOST} = $ENV{"HOST"} if $ENV{"HOST"}; 
	$DB{DATA}{HOST} = "localhost" if not $DB{DATA}{HOST};
        $DB{DATA}{HOST} = $ENV{"DBHOST"} if $ENV{"DBHOST"}; 
        $DB{WWW}{HOST} = $DB{DATA}{HOST};
        $DB{WWW}{HOST} = $ENV{"WWWDBHOST"} if $ENV{"WWWDBHOST"};
	$DB{DATA}{USER} = $ENV{"USER"} if $ENV{"USER"};
        $DB{DATA}{USER} = $ENV{"DBUSER"} if $ENV{"DBUSER"}; 
        $DB{WWW}{USER} = '';
        $DB{WWW}{USER} = $DB{DATA}{USER};
        $DB{WWW}{USER} = $ENV{"WWWDBUSER"} if $ENV{"WWWDBUSER"};
        $DB{DATA}{PASS} = 'unknown';
        $DB{DATA}{PASS} = $ENV{"DBPASS"} if $ENV{"DBPASS"}; 
        $DB{WWW}{PASS} = $DB{DATA}{PASS};
        $DB{WWW}{PASS} = $ENV{"WWWDBPASS"} if $ENV{"WWWDBPASS"};
	if ( (not $DB{DATA}{PASS}) || (not $DB{WWW}{PASS}) ) {
	    print STDERR "Please provide DB, and/or website DB credentials\n";
	    exit 1;
	}

	# drush website details ID
	# if drush uses a file it should have full path name
        my $drush = "\@bdp";
        my $debug = 0;
	my $Mode = "HTML";
	# fields to be changed
	# my $Drupal_fields = "body/und/*/value,body/und/*/safe_value";	# json fields Drupal
	my $Drupal_fields = "body/und/0/value,body/und/*/safe_value";	# json fields Drupal

	# handle command line keys
	while ( $ARGV[0] && ($ARGV[0] =~ /^-/) ) {
	    if ( $ARGV[0] =~ /^-d/ ) { $debug++; }
	    elsif ( $ARGV[0] =~ /^-j/ ) { $Mode = "JSON"; }
	    elsif ( $ARGV[0] =~ /^-(aqi|maxaqi|lki|maxlki|aqhi|caqi|(aqi)?min-?max)/i ) {
                my $v = lc($1); $v =~ s/(aqi)?min-?max/mmaqi/i;
                if( $v =~ /mmaqi/ ) { $AQIprint{mmlki} = 1; }
                $AQIprint{lc($v)} = 1;
                # comman line option lki/aqi implies max
                if( $v =~ /^(aqi|lki)$/i ) { $AQIprint{'max'.lc($v)} = 1; }
            }
	    elsif ( $ARGV[0] =~ /^-j/ ) { $Mode = "JSON"; }
	    elsif ( $ARGV[0] =~ /^-H/ ) { $Mode = "HTML"; }
	    elsif ( $ARGV[0] =~ /^-s/ ) { $Mode = "SQL"; }
	    elsif ( $ARGV[0] =~ /^-f$/ ) { $Drupal_fields = $ARGV[1]; shift @ARGV; }
            elsif ( $ARGV[0] =~ /^-f(.*)$/ ) { $forecast = $1; }
            elsif ( $ARGV[0] =~ /^-n$/ ) { $forecast = ''; }
	    elsif ( $ARGV[0] =~ /^-p/ ) {
		$scan_only = $ARGV[1]; shift @ARGV;
		$scan_only =~ s/,/|/g; $scan_only = "^($scan_only)\$";
	    }
	    elsif ( $ARGV[0] =~ /^-+h/ ) {
	        print STDERR "
Command parses the Drupal node, obtained via drush $drush entity-read command,
(node 3911 is default) and changes data measurement values from file arg1 (deflt
is stdin) and puts it back in  to the website via drush (json) or SQL DB access.
The pollutant values are averages of one day (24 hours) and if defined the minimal and maximum value of the day.

Optional arg2 = print will print collected data from KENMERKEN.data file.
Optional arg2 will define  the node number (dflt 3911) or input file to use.
If input file name or input node numner is defined the file will be
processed and converted to output.
Optional arg3 will define  output node number (default same as arg2)
or output filename.
The options will enable the use of eg LKI values in the HTML file generation.

Debugging: First create KENMERKEN.data: e.g. Day_results.sh KENMERKEN stationTBL1 .... stationTBLn
Use e.g. metingen.pl [-aqi -maxaqi -aqhi -lki -maxaqi -caqi] KENMERKEN.data  3911 output_file_name
to collect the HTML file into output_file_name.html with table values
from KENMERKEN.data.
Use e.g. metingen.pl KENMERKEN.data print to obtain a printout of what is found
in the data definition file KENMERKEN.data.

Switches:
  -d debug messages 1=table printout 2=HTML data on stdout, 3: sql query
  -s SQL website update, -j json website update,
  -H HTML output (default).
  -f fields fields to collect (dflt $Drupal_fields)..
  -p pollutions comma separated DB pollution names to use in html file table
     default: pm_10,pm_25
  -aqi add (max) AQI index values per pollution in table if defined
  -lki add (max) LKI index values of a pollution in table if defined
  -caqi add CAQI of (Europe) index values of pollutions in table if defined
  -aqhi add AQHI index values in table if defined
  -aqimin-max if AQI or LKI index values are defined add them to the table
  -fID add forecast PM2.5/weathertable with div id ID. Default: $forecast.
  -n disable local forecast ID. Adds AQICN for forecast. 

For arg 1 is file and arg 1 is 'print' just print file definitions.

TO DO:
There are some problems with drush: update gives errors, drush jason input works
only with perl json decoder for full node fields. Full node feed into drush
gives bundle errors.
Use only HTML (default) or SQL function.
"		;
		exit 0;
	    }
	    else { print STDERR "Unknown switch $ARGV[0]\n"; }
	    shift @ARGV;
	}
	my $Drupal_node; # 3911  default node number

        # generate json structure for node XXX Drupal dump
	my %data = ();	# collected data, limits and info for stations
	my %stations;	# per station per sensor real count of excess
	my @stationID;  # ordered list of station IDs
	my @sensor; 	#array with names of sensors

        my %mysql;
        # routine to query DB abd return results (ref to 2-D array of refs)
        # one row of values from one column out of the DB table
        use feature "state";    # some routines keep own state variables
	use DBI;		# need it for database queries
	use POSIX qw(locale_h);
        # use strict;
        # use warnings;
 
        binmode STDOUT, ":utf8";
        use utf8;
        #  
        use JSON;
 
	setlocale(LC_TIME, "nl_NL.UTF-8");

        sub Check_DB { # called only once from first call to query routine
	    my $type = shift;	# should be DATA or WWW
	    die "Not a correct type of database\n" if $type !~ /(WWW|DATA)/;
            if( defined $mysql{$type} ) { return 1; }
	    die "No database definition or DB credentials defined!"
                if (not $DB{$type}{USER})
                    || (not $DB{$type}{HOST})
                    || (not $DB{$type}{DB})
                    || (not $DB{$type}{PASS});
            return 0 if length( $DB{$type}{USER} ) < 1;
            return 0 if length( $DB{$type}{PASS} ) < 1;
            return 0 if length( $DB{$type}{DB}   ) < 1;
            return 0 if length( $DB{$type}{HOST} ) < 1;

	    $mysql{$type . 'Check'} = 0 if not defined $mysql{$type . 'Check'};
            $mysql{$type} = DBI->connect("DBI:mysql:".$DB{$type}{DB}.":".$DB{$type}{HOST},$DB{$type}{USER},$DB{$type}{PASS});
            if( not $mysql{$type} ) {
                if ( $mysql{$type . 'Check'} > 0 ) {
                    print STDERR "ERROR Cannot open mysql database ".$DB{$type}{DB}." on host ".$DB{$type}{HOST}.", with user ".$DB{$type}{USER}.", error: ". DBI->errstr."\n" ;
                    $mysql{$type} = 1; return 0;
                }
                exit 1;
                print STDERR "ERROR Cannot open mysql database ".$DB{$type}{DB}." on host ".$DB{$type}{HOST}.", with user ".$DB{$type}{USER}.", error: ". DBI->errstr."\nWill try to create it\n" ;
                $mysql{$type . 'Check'} = 1;
            }
            return 1;
        }
        sub query {
	    my $type = shift;
	    die ("Not correct type $type of DB used\n") if $type !~ /(DATA|WWW)/;
            return 0 if Check_DB($type) == 0;
            my $q = shift;
            if( (not defined $mysql{$type})
              || ($mysql{$type} == 1)
              || (not $q) ) {
                return undef;
            }
            print STDERR "SQL on $type DB: $q\n" if $debug > 2;
            my $sth = $mysql{$type}->prepare($q); $sth->execute();
            #  ref to 2 dimensional refs to array string values
            if ( $q =~ /^\s*(show|describe|select)/i ) {
                my $r = $sth->fetchall_arrayref();
                print STDERR "ERROR $type mysql query: $q with error:\n"
		    . DBI->errstr . "\n" if ( DBI->errstr &&  ($#{$r} < 0) );
                return undef unless $r->[0];
                $sth->finish(); return $r;
            } else { return undef; }
        }

	############## json input and output routines
	
	# comma separated list of files we are interested in
	# other fields are body/und/*/summary
	# seems to be more (internal) fields with safe_ prependend like safe_value
	# as well the fields changed and internal revision_timestamp
	# open a json data stream
        my $json;
	my $Drush_fields = "";
        sub Open_JSON {
          my $file = shift;
	  # get only that data we needs for this mode
	  # usually the fields are body/und/*/(value,safe_value)
	  my $fields = $Drupal_fields;
	  $fields =~ s/\/value.*/\/value/ if $Mode eq "HTML";
	  $fields =~ s/.*,// if $Mode eq "HTML";
	  $Drush_fields = $fields;	# remember for output
	  $fields = "--fields=" . $fields;
	  $fields = "";		######## some how the drush json is incorrect
	  # so we read all entity node data
          local $/; #Enable "slurp" mode
          my $fh;
          if ( -f $file ) {
	      print STDERR "Get template data via: file $file.\n";
              open $fh, "<$file"
	        || die ("Cannot find JSON file $file");
          } elsif ( $file =~ /^[1-9][0-9]+$/ ) {
	      $Drupal_node = $file;
	      # no revision_timestamp
	      print STDERR "Get template data via: \"drush $drush entity-read node $Drupal_node $fields --format=json\"\n";
              open $fh, "/homeOLD/teus/.composer/vendor/bin/drush $drush entity-read node $Drupal_node $fields --format=json |"
               || die ("Cannot call drush for json data $file for $drush\n");
          }
          die ("Cannot call drush for json data $file for $drush\n") if not $fh;
          $json = <$fh>;
          close $fh;
	  if ( $json =~ /^\{"[1-9][0-9]+":.*:\[\]}/ ) {
		die ("Cannot read in the data for fields $Drupal_fields\n");
	  }
	  return 1;
        }

        my $JSON_data;			# the perl struct with json data
	my %JSON_ptr;			# table with pntrs to string for Drupal_fields

	sub Walk {
	    my $ptr = shift;
	    my $fields = shift;
	    my $N;
	    return $ptr if not $fields;
	    my @tmp = split /\//, $fields;
	    my $field = shift @tmp;
	    my $remain = join "/", @tmp;
	    return undef if (ref($ptr) eq "HASH") && (not defined $ptr->{$field});
	    # array element * is 0 (zero) here
	    $field = 0 if (ref($ptr) eq "ARRAY") && ($field eq "*") ;
	    if ( (ref($ptr) eq "ARRAY") && (defined $ptr->[$field]) ) {
	        return \$ptr->[$field] if not $remain;
		$N = \%{$ptr->[$field]} if ref($ptr->[$field]) eq "HASH";
		$N = \@{$ptr->[$field]} if ref($ptr->[$field]) eq "ARRAY";
	    } elsif( (ref($ptr) eq "HASH") && (defined $ptr->{$field}) ) {
	        return \$ptr->{$field} if not $remain;
		$N = \%{$ptr->{$field}} if ref($ptr->{$field}) eq "HASH";
		$N = \@{$ptr->{$field}} if ref($ptr->{$field}) eq "ARRAY";
	    } else { return undef; }
	    return Walk($N, $remain) if $remain;
	    return undef;
	}

	# get the address of the Drupal field string
	sub Get_Ptr {
	    return undef if ref($JSON_data) ne "HASH";
            #my $ptr = \%{$JSON_data->{$Drupal_node}};
	    return Walk ( \%{$JSON_data->{$Drupal_node}}, shift);
	}

        my %IN;				# arrays of input json data
        my $Data_Field;			# which input modus we are
        sub GetIt {
	    state @Fields;
            # Field Types: $Drupal_fields comma seperated
	    # to do: use shift iso ptr as index in array
            if ( not defined $JSON_data ) {
	        return "" if not $json;
                $JSON_data = decode_json($json)
                    || die ("Cannot do JSON decode, not a json file\n");
	        undef $json;
		@Fields = split /,/, $Drush_fields; # only the fields we have read
		for my $Field (@Fields) {
		   $IN{$Field}{data} = ();
		   next if ($Field !~ /\/value/) && ($Mode eq "HTML");
		   my $ptr = Get_Ptr ( $Field );
		   if ( not $ptr ) {
		       print STDERR "WARNING: Cannot find data for field $Field\n";
		       my $a = ""; $ptr = \$a;
		   }
	           my @t = split /\n/, $$ptr;
                   $IN{$Field}{data} = \@t;
                   $$ptr = "";
		}
		$Data_Field = shift @Fields;
            }
	    while ( $IN{$Data_Field}{data} && $#{$IN{$Data_Field}{data}} < 0 ) {
		return "" if $#Fields < 0;
		$Data_Field = shift @Fields;
	    }
            my $strg = shift @{$IN{$Data_Field}{data}};
	    $strg = "" if not $strg;
            return $strg . "\n";
        }

	# add string (arg1) to json data struct default field Data_Field
	# optional arg2 for other fields
use HTML::Entities;
	sub PutIt {
	    state $CUR_field = "XXX";
	    state $CUR_data;
            # Field Types: value|summary|format|safe_value|safe_summary
	    # changed|revision_timestamp
	    my $strg = shift;
	    my $field = shift;
	    # omit unicode chars as micro and 3 super
	    $strg = encode_entities($strg,"\200-\x{FFFF}");
	    print STDOUT $strg if ($debug == 2) && ($Data_Field =~ /\/value/);
	    $field = $Data_Field if not $field;
	    return 0 if (not $field) || (not $JSON_data);
	    return 0 if $field !~ /(\/value|\/safe_value|changed|revision_timestamp)/;
	    if ( $CUR_field ne $field ) {
	        my $CUR = Get_Ptr($field);
		return 0 if not defined $CUR;
		$CUR_field = $field; $CUR_data = $CUR;
	    }
	    $$CUR_data .= $strg;
	    return 1;
	}
		
	# close and write to file in HTML format
	sub Close_HTML {
	    return 0 if (not $Data_Field) || (not %IN) || (not $JSON_data);
            my $file = shift;
	    # return 0 if $Mode ne "HTML";
	    my $field = $Drupal_fields;
	    $field =~ s/\/value.*/\/value/; $field =~ s/,.*//;
	    my $data = Get_Ptr($field);
	    $file = "node$file" if $file =~ /^[0-9]+$/;
	    open $fh, ">$file.html";
	    print $fh $$data;
	    print STDERR "New HTML file $file.html for node $Drupal_node.\n";
	    return close $fh;
	}

	# close and write to file or database in sql format
	sub Close_SQL {
	    return 0 if (not $Data_Field) || (not %IN) || (not $JSON_data);
	    my $file = shift;
	    my $time = time;
	    if ( $Mode eq "JSON" ) {
		return Close_JSON ( $file );
	    }
	    return Close_HTML($file) if $Mode eq "HTML";
	    my $SQL = 1;
	    $SQL = 0 if $file !~ /^[0-9]+$/;
    print STDERR "MYSQL update query not tested yet, print SQL on file.\n";
    $file = $Drupal_node; $SQL = 0;	# force not to use SQL
    $file = "node$file.sql" if $file =~ /^[0-9]+$/;
	    my %My_Table = (
		value => "field_data_body",
		field_data_body => "body_value",
		safe_value => "field_revision_body",
	        field_revision_body => "body_value"
	    );
    	    my $fh;
	    if ( not $SQL ) {
    		open $fh, ">$file";
    		print STDERR "New SQL data for node $Drupal_node on file $file.\n";
    		print $fh "USE $DB{WWW}{DB};\n";
	    } else {
	#	$fh = query('WWW',"USE $DB{WWW}{DB}");
	    }
	    for my $field (split /,/, $Drupal_fields) {
	        my $data = Get_Ptr($field);
		if ( defined $data ) {
		    $$data =~ s/\'/\\\'/;
	            $$data =~ s/\n/\\n/g; $$data =~ s/\t/\\t/g;
		}
		$field =~ s/.*\/([a-z_]+)$/$1/;
		next if not $My_Table{$field};
	        if ( not $SQL ) {
		    print $fh "LOCK TABLES $My_Table{$field} WRITE;\n";
		    print $fh "UPDATE $My_Table{$field} SET $My_Table{$My_Table{$field}} =\n'";
		    print $fh $$data;
		    print $fh "'\nWHERE entity_id = $Drupal_node;\n";
		    print $fh "UNLOCK TABLES;\n";
		    next;
	        } else {
	    #	$fh = query('DATA', "UPDATE $My_Table{$field}
	    #	    SET $My_Table{$My_Table{$field}} = \'$$data\'
	    #	    WHERE entity_id = $file") if not $debug;
	    #	$fh = query('WWW', "UPDATE node SET changed = $time
	    #	    WHERE node = $Drupal_node") if not $debug;
		    next;
	        }
	    }
	    print  $fh "UPDATE node SET changed = $time WHERE node = $file;\n";
	    close $fh;
	 #  $fh = query('DATA',"USE $DB{DATA}{DB}") if not $debug;
	 #  print STDERR "Node $file updated on database $DB{WWW}{DB}\n" if not $debug;
	    print STDERR "Node $file NOT updated on database $DB{WWW}{DB}\n" if $debug;
	    undef $JSON_data; undef %IN; undef $Data_Field;
	    return 1;
	}

	# close and write out json struct   
	sub Close_JSON {
	    return 0 if (not $Data_Field) || (not %IN) || (not $JSON_data);
	    my $file = shift;
	    my $fields = $Drush_fields; $fields = "--fields=$fields" if $fields;
	    ######### see open section: somehow drush cannot handle parts
	    $fields = "";	########################################
	    if ( $Mode ne "JSON" ) {
		return Close_SQL ( $file );
	    }
	    if ( $file =~ /^[1-9][0-9]+$/ ) {
		# no revision_timestamp
		# to do: this seems not to work for some reason
		print STDERR "New data for node $Drupal_node pushed via drush $drush $Drush_fields.\n";
	print STDERR "Abort: drush entity-update not functioning yet\n";
        print STDERR "Use: cat node$file.json | drush --pipe $drush entity-update node $file $fields --json-input=-\n";
	$file = ">node$file.json";
		# FIELDS PATH ARE NOT YET SUPPORTED
	#	$file = "| drush --pipe $drush entity-update node $file $fields --json-input=-";
	    } else { $file = ">$file.json" ; }
	    my $fh;
	    open $fh, $file;
	    # seems drush does not allow node number in json?
	    $JSON_data = JSON::XS->new->ascii (1)->encode($JSON_data->{$Drupal_node});
	    # seems Drupal needs all Unicode chars in encoded
	    $JSON_data = encode_entities($JSON_data,"\030-\047\060-\066\200-\x{FFFF}");
	    #$JSON_data =~ s/</\\u003c/g; $JSON_data =~ s/>/\\u003e/g;
	    #$JSON_data =~ s/\&/\\u0026/g;
	    print $fh $JSON_data;
	    close $fh; undef $JSON_data; undef %IN; undef $Data_Field;
	    print STDERR "New data for node $Drupal_node in JSON format file $file.\n";
	    return 1;
	}

	# # get missing data from database
	sub CompleteData {
	    state $OneError = 0;
	    # reset no sensors measured
	    for my $ID (@stationID) {
                for ( my $i = 0; $i <= $#sensor; $i++ ) {
		    $stations{$ID}{$sensor[$i]} = -1
			if not defined $stations{$ID}{$sensor[$i]};
		}
	    }		
	    foreach my $key (keys %data) {
                next if $key !~ /^20[0-9]{2}-[01][0-9]-[0-3][0-9]$/ ;
		for my $ID (@stationID) {
                    for ( my $i = 0; $i <= $#sensor; $i++ ) {
	              if( (not defined $data{$key}{$ID})
                            || (not defined $data{$key}{$ID}[$i])
                            || (not $data{$key}{$ID}[$i]) ) {
			  # if column define calculate day avg, if not -1
			  # is the sensor in the DB?
			  $data{$key}{$ID}[$i] = -1;
			  $stations{$ID}{$sensor[$i]} = -1
			     if not defined $stations{$ID}{$sensor[$i]};
                          my $qry = query('DATA',
			        "SELECT count(*)
				    FROM information_schema.columns
				    WHERE table_name = '$data{table}{$ID}'
				    AND column_name = '$sensor[$i]'");
			  if( (defined $qry) && ($#{$qry} >= 0)
			    && ($#{$qry->[0]} >= 0)
			    && ($qry->[0][0] > 0)
			    ) {  # yes it is
			      $data{$key}{$ID}[$i] = 0;
			      $qry = query ('DATA',
				      "SELECT round(avg($sensor[$i]))
                                        FROM $data{table}{$ID}
                                        WHERE date(datum) = \"$key\" ");
			       if( (defined $qry) && ($#{$qry} >= 0)
				 && ($#{$qry->[0]} >= 0) ) {
				   if( defined $qry->[0][0] ) {
                                       $data{$key}{$ID}[$i] = $qry->[0][0];
				   } else {
					$data{$key}{$ID}[$i] = 0;
				   }
                                   $data{$key}{$ID}[$i] = 0
					if ($data{$key}{$ID}[$i] <= 1)
				    && ($data{$key}{$ID}[$i] > 0);
			       }
			  } else {
				$stations{$ID}{$sensor[$i]} = -1;
			  }
		          $stations{$ID}{$sensor[$i]}++
			       if ($stations{$ID}{$sensor[$i]} >= 0)
			           && ($data{$key}{$ID}[$i] >= $data{norm}{MAX}{$sensor[$i]});
                      }
		      if( int($data{$key}{$ID}[$i]) >= $data{norm}{MAX}{$sensor[$i]} ) {
		          $data{$key}{$ID}[$i] =
			    sprintf("%d!", int($data{$key}{$ID}[$i]));
		      } elsif (int($data{$key}{$ID}[$i]) < 0 ) {
		          $data{$key}{$ID}[$i] = "?";
		      } elsif ( not int($data{$key}{$ID}[$i]) ) {
		          $data{$key}{$ID}[$i] = "-";
		      }
		      # count exceeding limits # of realy measured days
		      # $data{norm}{MAXyr|MAX|MAXdays}{sensor}
                    }
		}
            }
	    return 1
	}

	sub Get_defs {
	    my $input = shift;
            open INPUT, "<$input" or die "Cannot open datafile $input\n";
	    while( <INPUT> ) {
	        chop;
	        next if /#/ ;
	        if ( /^9999[^;]*;(.*):([^:]*)/ ) {
		    my $station = $1;
		    my $t = $2; $t = lc($t); $t =~ s/pm_?/pm_/g;
		    $t =~ s/\.//g;
		    if ( $station =~ /station/ ) {
		        @sensor = split /[,\s]/, $t;
		    } else {
		        # keep ordered list of station table names
		        if ( not defined $stations{$station}  ) {
		            $stationID[++$#stationID] = $station;
		        }
			for( my $i = 0; $i <= $#sensor; $i++ ) {
			    $stations{$station}{$sensor[$i]} = -1;
			}
			foreach my $se (split /[,\s]/, $t) {
			    # $stations{$station}{sensor}: 0 is in DB, -1 is not,
			    # >0 has cnt values on some date
			    $stations{$station}{$se} = 0 ;
			}
		    }
	        } elsif ( /^(20[0-9][0-9]-[01][0-9]-[0-3][0-9]);([^:]+):([a-z0-9_]+)=([0-9\.]+)\s/ ) {
		    my $datum = $1; my $tbl = $2; s/^[^:]+://;
	    	    my @senses=();
	    	    foreach my $el (split /[,\s]/) {
			if ( $el =~ /^([a-z_0-9]+)=([0-9\.]+)$/ ) {
		    	    my $sens=$1; my $val=$2;
			    $sens = lc($sens); $sens =~ s/pm_?/pm_/;
		    	    for( my $i=0; $i <= $#sensor; $i++ ) {
			        if( $sensor[$i] eq $sens ) { $senses[$i] = $val; last; }
		    	    }
			}
	    	    }
		    # keep ordered list of station table names
		    if ( not defined $stations{$tbl}  ) {
		        $stationID[++$#stationID] = $tbl;
		    }
		    for (my $i = 0; $i <= $#sensor; $i++ ) {
			$stations{$tbl}{$sensor[$i]} = -1
			    if not defined $stations{$tbl}{$sensor[$i]};
			$data{$datum}{$tbl}[$i] = "";
			next if not defined $senses[$i];
			$stations{$tbl}{$sensor[$i]} = 0
			    if ($stations{$tbl}{$sensor[$i]} < 0);
			$data{$datum}{$tbl}[$i] = $senses[$i];
			# mark there is a value
			#$stations{$tbl}{$sensor[$i]}++
			#    if ($stations{$tbl}{$sensor[$i]} >=0) && ($senses[$i] > 0);
		    }
	        } elsif ( /^norm:/ ) {
		    if( /^norm:(.*)_(MAX|MAXyr|MAXdays)=([0-9]+)$/ ) {
			my $f1=$1; my $f2=$2; my $f3 = $3; $f1=lc($f1);
			$f1 =~ s/^pm_?/pm_/; $f1 =~ s/\.//;
			# $data{norm}{MAXyr|MAX|MAXdays}{pollutant}=value
			$data{norm}{$f2}{$f1}=$f3;
		    }
		} elsif ( /^([A-Za-z0-9]+)_(avg|days|today|diff|lastday):([^=]+)=(.*)$/ ) {
		    # data{avg,days,today,lastday}{station}{'pol'}=value
		    my $p = lc($1); my $t = lc($2); my $s = $3; my $v = $4;
		    $p =~ s/^pm_?/pm_/; $p =~ s/pm_2\./pm_2/;
		    if( $p =~ /(avg|today)/ ) {
		        my $v1 = $v; $v1 =~ s/[^0-9\.]//g;
		        $v = 0 if $v1 <= 1;	# ???
		    } elsif ( $p =~ /(lastday)/ ) {
			$v = "" if $v !~ /[a-z]+\s[0-9]+\s[a-z]+\s20[0-9][0-9]/i;
		    }
		    $data{$t}{$s}{$p}=$v;
                } elsif ( /^([A-Za-z0-9]+)_(LKI|AQI|CAQI)([a-z]*):([^=]+)=(.*)$/ ) {
                    # LKI AQI CAQI may have per pol an index value if value > 0
		    # data{lki,aqi with ext (color|gom)}{station}{'pol'}=value
		    my $p = lc($1); my $t = lc($2);
                    my $e = ''; $e = $3 if defined $3;
                    my $s = $4; my $v = $5;
		    $p =~ s/^pm_?/pm_/; $p =~ s/pm_2\./pm_2/;
		    $data{$t.$e}{$s}{$p}=$v;
                    # for index AQI LKI CAQI keep track of min and max value
                    if ( (not $e) && ($v > 0) ) {
                          if ( (not defined $data{$t}{$s}{'max'}{'val'})
                            || (not defined $data{$t}{$s}{'min'}{'val'})
                          ) {
                            $data{$t}{$s}{'max'}{'val'} = $v;
                            $data{$t}{$s}{'max'}{'pol'} = $p;
                            $data{$t}{$s}{'min'}{'val'} = $v;
                            $data{$t}{$s}{'min'}{'pol'} = $p;
                          } elsif ( (defined $data{$t}{$s}{'max'}{'val'})
                                && ( $v > $data{$t}{$s}{'max'}{'val'}) ) {
                            $data{$t}{$s}{'max'}{'val'} = $v;
                            $data{$t}{$s}{'max'}{'pol'} = $p;
                          } elsif ( (defined $data{$t}{$s}{'min'}{'val'})
                                && ($v < $data{$t}{$s}{'min'}{'val'}) ) {
                            $data{$t}{$s}{'min'}{'val'} = $v;
                            $data{$t}{$s}{'min'}{'pol'} = $p;
                          }
		          $AQIprint{$t}++
		            if (defined $AQIprint{$t}) && ($AQIprint{$t} > 0);
                    }
		} elsif ( /^(date|location|name|identity|table|smog):([^=]+)=(.*)$/i ) {
	        # $data{date,name,location,table,aqhi...}{station}=value
                    my $v = $3; my $t = lc($1);  my $s = $2;
                    $data{$t}{$s}=$v;
                } elsif ( /^MM(Max|Min|Avg|Day|Gom|Qual|Col)(AQI|LKI|CAQI|AQHI):([^=]+)=(.*)$/i ) {
                    # the aqi average is taken from avg per day per pollutant
                    # this value may exceed the max of aqi per hour for one day
                    # Gom|Qual|Col are values from day avg index
                    my $aqi = 'mm-'.lc($2); my $station = $3;
                    my $v = $4; my $min_max = lc($1);
                    if( $v !~ /\s/ ) {
                        $data{$aqi}{$station}{$min_max}=$v;
                    } else {
                        my $pol_val = $v; $pol_val =~ s/^[0-9\.]+\s+//;
                        $v =~ s/^([0-9\.]+)\s.*/$1/;
                        $data{$aqi}{$station}{$min_max}{'pol_val'}=$pol_val;
                        $data{$aqi}{$station}{$min_max}{'index'}=$v;
                    }
                } elsif ( /^(Max)?(AQI|LKI|CAQI|AQHI)([A-Za-z]*):([^=]+)=(.*)$/i ) {
	        # $data{date,name,location,table,aqhi...}{station}=value
                    my $v = $5; my $t = lc($2);  my $e = '';  my $s = $4;
                    $e = $3 if defined $3;
                    $t = 'max' . $t if defined $1;
                    $t = $t . $e;
                    $data{$t}{$s}=$v;
                    #print STDERR "\"$_\" -> data{$t}{$s}=$v\n";
                    if ( ($t =~ /^(LKI|AQI)$/i) && ($v > 0) ) {
                        # should not happen
                        $AQIprint{$t}++
                            if (defined $AQIprint{$t}) && ($AQIprint{$t} > 0);
                        if ( (not defined $data{$t}{$s}{'max'})
                            || (not defined $data{$t}{$s}{'min'}) ) {
                            $data{$t}{$s}{'max'} = $data{$t}{$s}{'min'} = $v;
                        } elsif ( $v > $data{$t}{$s}{'max'} ) {
                            $data{$t}{$s}{'max'} = $v;
                        } elsif ( $v < $data{$t}{$s}{'min'} ) {
                            $data{$t}{$s}{'min'} = $v ;
                        }
                    }
                    if( (not $e) && ($v > 0 ) ) {
		        $AQIprint{$t}++
			    if (defined $AQIprint{$t}) && ($AQIprint{$t} > 0);
                    }
	        } else {
                    print STDERR "skip: $_\n" if $debug;
                }
	    }
	    close INPUT;
	    CompleteData;
	    return 1;
	}

	# for AQI index or AQHI index calculation we need this script
	#require "/webdata/luchtmetingen/AQI.pl";

	use POSIX qw(strftime);
	use Time::Piece;                # need to parse time strings
	use DateTime::Locale;
	DateTime::Locale->load("nl_NL");
	
	sub NL_time {
	    my $loc = DateTime::Locale->load("nl_NL");
	    my $tt = shift; return "" if not $tt;
	    my $timing = Time::Piece->strptime($tt . " 0.29 +0100", "%Y-%m-%d %H.%M %z")->epoch;
	    return strftime("%a %e %b %Y", localtime($timing));
	}
	
	sub Print_Table {
            my @Stations = @_;
            if ( $#Stations < 0 ) {
                @Stations = @stationID;
            } else { # check first if stations from argument have defined data
                my @list = ();
                for  my $id ( @Stations ) {
                    my @CNTr = grep /^$id$/, @stationID;
                    if ( $#CNTr < 0 ) {
                        print STDERR "Cannot find data for station $id\n";
                        next;
                    }
                    push @list, $id;
                }
                return 0 if $#list < 0;
                @Stations = @list;
            }
	    print "Meetstations  ";
	    for my $ID (@Stations) {
	        print "\t$ID";
	    }
	    print "\n";
	    print "Emissiestoffen in DB: ";
	    {
		my $j = join ", ", @sensor; $j = uc($j);
		$j =~ s/PM_/PM/g; $j =~ s/PM25/PM2.5/;
	        print "\t$j";
	    }
	    print "\n";
	    for my $ID (@Stations) {
	        print "Station $ID:\n";
                if ( defined $data{smog}{$ID} ) {
                    printf("\tSmog: %salert\n", ($data{smog}{$ID} > 0? '' : 'geen '));
                }
		for( my $i = 0; $i <= $#sensor; $i++ ) {
		    my $SENS = $sensor[$i]; my $pol = uc($SENS);
		    next if $stations{$ID}{$SENS} < 0;
		    $pol =~ s/PM_/PM/; $pol =~ s/^PM2/PM2./;
		    next if $stations{$ID}{$SENS} < 0;
		    printf("\t%s: %s µg/m³ gemiddelde over laatste 365 dagen\n",
			$pol, $data{avg}{$ID}{$SENS})
			if ( defined $data{avg}{$ID}{$SENS} )
			     && ($data{avg}{$ID}{$SENS} > 0);
		    if ( (defined $data{today}{$ID}{$SENS})
			     && ($data{today}{$ID}{$SENS} > 0) ) {
		        printf("\t%s: op %s daggemiddelde %s µg/m³",
			    $pol, $data{date}{TODAY}, $data{today}{$ID}{$SENS});
			if ( defined $data{diff}{$ID}{$SENS} ) {
			  printf(", stijgt met %d µg/m³", $data{diff}{$ID}{$SENS})
			    if $data{diff}{$ID}{$SENS} > 0;
			  printf(", daalt met %d µg/m³", $data{diff}{$ID}{$SENS})
			    if $data{diff}{$ID}{$SENS} < 0;
			  print(" en blijft stabiel")
			    if $data{diff}{$ID}{$SENS} == 0;
			}
		    }
		    print "\n";
		    my $val = $data{days}{$ID}{$SENS}
			if defined $data{days}{$ID}{$SENS};
		    $val =~ s/[^0-9\.]//g;
		    if ( (defined $data{days}{$ID}{$SENS}) && ($val> 0) ){
	                printf("\t%s: %s aantal dagen met overschrijdingen (%s%s)\n",
			    $pol, $data{days}{$ID}{$SENS},
			    ($data{norm}{MAX}{$SENS} < 99999 ?
			      "max $data{norm}{MAX}{$SENS} µg/m³"
			    : "geen max avg"),
			    ($data{norm}{MAXdays}{$SENS} >= 365 ?
				""
			      : ", maximaal $data{norm}{MAXdays}{$SENS} dagen")
			);
		    } else {
			printf("\t%s: geen overschrijdingen (%s)\n",
			    $pol,
			    ($data{norm}{MAXdays}{$SENS} >= 365?
			       "geen maximale norm"
			     : "max $data{norm}{MAX}{$SENS}  µg/m³, max $data{norm}{MAXdays}{$SENS} dagen"));
		    }
		}
                foreach my $aqi ('aqi', 'maxaqi', 'aqhi', 'lki', 'maxlki', 'caqi' ) {
                    if ( defined $data{$aqi}{$ID} ) {
                        if ( $aqi =~ /^(aqi|lki)$/ ) {
                            my $frst = 0;
                            foreach my $pol ( keys %{$data{$aqi}{$ID}} ) {
                                if ( $pol =~ /^(max|min)$/ ) {
                                    next;
                                } 
                                if ($data{$aqi}{$ID}{$pol} > 0) {
                                    printf("\t%s index:",uc($aqi))
                                        if not $frst++;
                                    printf(" %3.1f (%s)", $data{$aqi}{$ID}{$pol}, $pol);
                                }
                            }
                            if ( (defined $data{$aqi}{$ID}{'min'}{'val'})
                                && (defined $data{$aqi}{$ID}{'max'}{'val'})
                               ) {
                                if ( $data{$aqi}{$ID}{'min'}{'pol'} ne $data{$aqi}{$ID}{'max'}{'pol'} ) {
                                    my $pmin = uc($data{$aqi}{$ID}{'min'}{'pol'});
                                    my $pmax = uc($data{$aqi}{$ID}{'max'}{'pol'});
                                    $pmin =~ s/PM_/PM/; $pmin =~ s/PM25/PM2.5/;
                                    $pmin = lc($pmin) if $pmin =~ /^([A-Z]{3,}?|[0-9])/;
                                    $pmax =~ s/PM_/PM/; $pmax =~ s/PM25/PM2.5/;
                                    $pmax = lc($pmax) if $pmax =~ /^([A-Z]{3,}?|[0-9])/;
                                    printf("\n\t%s min-max: %3.1f (%s,%s) - max %3.1f (%s,%s)",
                                        uc($aqi),
                                        $data{$aqi}{$ID}{'min'}{'val'}, $pmin,
                                        $data{$aqi.'qual'}{$ID}{$data{$aqi}{$ID}{'min'}{'pol'}},
                                        $data{$aqi}{$ID}{'max'}{'val'}, $pmax,
                                        $data{$aqi.'qual'}{$ID}{$data{$aqi}{$ID}{'max'}{'pol'}})
                                        if ($data{$aqi}{$ID}{'max'}{'val'} > 0)
                                            && ($data{$aqi}{$ID}{'max'}{'val'} > 0);
                                }
                            }
                            print "\n" if $frst;
                        } elsif ( $data{$aqi}{$ID} > 0 ) {
                            printf("\t%s index: %3.1f", uc($aqi), $data{$aqi}{$ID});
		            printf(", kleur %s", $data{$aqi.'color'}{$ID})
		                if defined $data{$aqi.'color'}{$ID};
		            printf(", melding %s", $data{$aqi.'qual'}{$ID})
		                if (defined $data{$aqi.'qual'}{$ID}) && ($data{$aqi.'qual'}{$ID});
		            print("\n");
		            printf("\t%s GoM:%s\n", uc($aqi), $data{$aqi.'gom'}{$ID})
		                if $debug && (defined $data{$aqi.'gom'}{$ID}) && $data{$aqi.'gom'}{$ID};
                        }
                    }
                }
	    }
	    print "\n";
	    foreach my $key (sort { $b cmp $a ; } keys %data) {
	        next if $key !~ /^20[0-9][0-9]-[01][0-9]-[0-3][0-9]$/ ;
	        print "    " . NL_time($key) . "\n";
	        for my $ID (@Stations) {
		    print "\t${ID}\t";
                    my $aqi = '';
		    if ( defined $data{$key}{$ID} ) {
			for( my $i = 0; $i <= $#sensor; $i++) {
			    my $S = uc($sensor[$i]); $S =~ s/PM_/PM/; $S =~ s/PM25/PM2.5/;
			    printf(" %s=%s", $S, $data{$key}{$ID}[$i]);
                            next if $data{$key}{$ID}[$i] =~ /[\-\?]/;
                            $aqi .= ' ' if $aqi;
                            $aqi .= "$S $data{$key}{$ID}[$i]";
			}
		    }
                    # need to extend print line with aqi index value?
                    my $frst = 0;
                    next if not $aqi;
                    foreach my $AQI ( keys %AQIprint ) {
                        next if $AQIprint{$AQI} < 1 ;
                        my @AQIrts = (); $aqi =~ s/\!//g;
                        if ( $AQI =~ /maxaqi/ ){
                            @AQIrts = maxAQI($aqi); }
                        elsif ( $AQI =~ /aqhi/ ){
                            @AQIrts = AQHI($aqi) ;  }
                        elsif ( $AQI =~ /maxlki/ ){
                            @AQIrts = maxLKI($aqi); }
                        elsif ( $AQI =~ /caqi/ ){
                            @AQIrts = maxCAQI($aqi); }
                        else { next ; }
                        next if $AQIrts[0] < 0.05;
                        my $myAQI = uc($AQI); $myAQI =~ s/MAX//;
                        printf("%s%s index %3.1f (%2.1f %s)",
                            ( ($frst++ % 2) ? ' ' : "\n\t\t"),
                            $myAQI, $AQIrts[0], $AQIrts[0]*10/$AQItable{$AQI}{range},$AQIrts[2]);
                    }
	            print "\n";
		}
	    }
	    print "\n";
	    return 0;
	}

	sub SKIP2TD {
	    my $_; my $tr = 0;
	    while( $_ = GetIt ) {
		last if not $_;
		next if (not /<\/tr>/) && (not $tr);
		$tr++;
                if ( /\s[a-z]{2}\s+[0-9]{1,2}\s+[a-z]{3,}\s+20[0-9]{2}/i ) {
                    s/\s[a-z]{2}\s+[0-9]+\s+[a-z]{3,}\s+20[0-9]{2}/ $data{date}{YESTERDAY}/i;
                }
	        PutIt $_;
		last if /<\/td>/ ;
            }
	    return "" if not $_;
	    return $_;
	}

        # calculate best contrating text color on background colour
        sub YIG{
            my $col = shift; $col =~ s/^#//;
            return ((hex(substr($col,0,2))*299)+(hex(substr($col,2,2))*587)+(hex(substr($col,4,2))*114))/1000;
        }

        sub rgbToHex {
            if( $_[0] =~ /rgb\s*\(\s*([0-9]+)\s*,\s*([0-9]+)\s*,\s*([0-9]+)\s*\)/ ) {
                return sprintf("%2.2X%2.2X%2.2X",$1,$2,$3);
            } else { return $_[0]; }
        }

        use Graphics::ColorNames;
        sub Col2Hex {
            my $col = shift; return '000000' if not defined $col;
            return $col if $col =~ /^#?[0-9a-f]{6}$/i;
            return '000000' if $col !~ /^[a-z]+$/i;
            my $po = new Graphics::ColorNames;
            return $po->hex($col);
        }

        use Color::Scheme;
        sub Text_col {
            my $bg = shift;
            my @colors = @_;
            @colors = ('000000', 'ffffff') if not defined $_[0];
            $colors[0] =~ s/^/#/ if $colors[0] =~ /^[0-9a-f]{6}$/i;
            $colors[1] =~ s/^/#/ if $colors[1] =~ /^[0-9a-f]{6}$/i;
            return $colors[0] if not defined $bg;
            $bg =~ s/^#//;
            $bg =~ s/([0-9a-f])([0-9a-f])([0-9a-f])/$1$1$2$2$3$3/
                if( $bg =~ /^[0-9a-f]{3}$/ );
            $bg = rgbToHex($bg) if $bg =~ /rgb\([0-9,\s]+\)/i;
            $bg = Col2Hex($bg) if $bg !~ /^#?[0-9a-f]{6}$/i;
            return $colors[0] if (not defined $bg) or ($bg !~ /^[0-9a-f]{6}$/i);
            return (YIG( $bg ) >= 128 ) ? $colors[0] : $colors[1];
        }

        sub Text_col_schemed {
            my $col = shift; my $set = shift;
            $set = 3 if (not defined $set) || ($set < 0);
            return '#000000' if (not defined $col) || (not $col);
            $col =~ s/^#//;
            $col =~ s/([0-9a-f])([0-9a-f])([0-9a-f])/$1$1$2$2$3$3/
                if( $col =~ /^[0-9a-f]{3}$/ );
            $col = rgbToHex($col) if $col =~ /rgb\([0-9,\s]+\)/i;
            $col = Col2Hex($col) if $col !~ /^#?[0-9a-f]{6}$/i;
            return '#000000' if (not defined $col) && ($col !~ /^[0-9a-f]{6}$/);;
            my $scheme = Color::Scheme->new();
            $scheme->from_hex($col);
            $scheme->scheme('analogic');
            $scheme->distance(0.25);
            $scheme->add_complement(1);
            $scheme->variation('hard');
            $scheme->web_safe(1);
            my @setlist = @{$scheme->colorset};
            $set = $#setlist if $set > $#setlist;
            return Text_col($col,$setlist[$set][1],$setlist[$set][2]);
        }

	sub Parse_Template {
	    my $input = shift;
	    return 0 if not $input;
	    my $output = shift;
	    $output = $input . "drupal" if not $output;
	    # define tab and end of line on output
	    my $T = "\t"; my $E = "\r"; # denote first and compact part
	    my @NRpolsa = split /\|/, $scan_only; my $NRpols = $#NRpolsa + 1; undef @NRpolsa;
	    $data{date}{TODAY} = strftime("%a %e %B %Y", localtime(time))
		if (not defined $data{date}{TODAY})
                    ||  (not $data{date}{TODAY});
	    my $no_table = 1;	# are we not in table context?
	    $data{date}{YESTERDAY} = strftime("%a %e %b %Y", localtime(time - 24*60*60))
		if (not defined $data{date}{YESTERDAY})
                    || (not $data{date}{YESTERDAY});
	    return 0 if not Open_JSON ( $input ); # get the input in memory
	    while( $_ = GetIt ) {
		last if not $_;
		# second part does not have tab spaces and \r
		if( $Data_Field =~ /\/safe_value/ ) {
		    $T = ""; $E = "";
		    last if $Mode eq "HTML";
		}
		# the last measurement date is in the first part
	 	# first time pattern is date of writing
		$no_table = 0 if /<table/i;
		$no_table = 1 if /<\/table/i;
		my $LastDsecs = 0; my $LastD = $data{date}{YESTERDAY};
		my $Recent = $data{date}{YESTERDAY};
		$Recent =~ s/^(ma|di|wo|do|vr|za|zo)\s+//;
                $Recent =~ s/mei/May/; $Recent =~ s/mrt/Mar/; $Recent =~ s/okt/Oct/;
		$Recent = Time::Piece->strptime($Recent . "  0.29 +0100", "%e %b %Y %H.%M %z")->epoch
                        if $Recent =~ /[0-9]+\s.*\s20[0-9]{2}/;
		$Recent = Time::Piece->strptime($Recent . " 0.29 +0100", "%Y-%m-%d %H.%M %z")->epoch
                        if $Recent =~ /20[0-9]{2}\-[0-9]+\-[0-9]+$/;
		$Recent -= 3*24*60*60;	# within 3 days from yesterday
                my $add = 0;
                for my $item ('aqi','maxaqi','caqi','lki','maxlki','aqhi') {
                    $add++ if (defined $AQIprint{$item}) && ($AQIprint{$item} > 1);
                }
		if( /Voor het verloop van fijnstof emissies/ && $add ) {
		    my $Secs = 0; my $Fnd_one = 0;
		    my $loc = ""; my $max = 0; my $maxID = ""; my $name = "";
                    foreach my $TBL ( sort { $AQItable{$a}{pri} <=> $AQItable{$b}{pri} } keys %AQItable ) {
                      my $mm_aqi = 'mm-'.$TBL; $mm_aqi =~ s/max//;
		      foreach my $ID ( sort keys %{$data{$AQItable{$TBL}{maxAQ}}} ) {
                        my $mm_def = 0;
                        next if $TBL =~ /^aqi$/;        # only on max index 
			next if (not defined $data{$AQItable{$TBL}{maxLD}}{$ID})
                           || (not $data{$AQItable{$TBL}{maxLD}}{$ID}); 
			$Secs = $data{$AQItable{$TBL}{maxLD}}{$ID};
			$Secs =~ s/^(ma|di|wo|do|vr|za|zo)\s+//;
                        $Secs =~ s/mei/May/; $Secs =~ s/mrt/Mar/; $Secs =~ s/okt/Oct/;
			$Secs = Time::Piece->strptime($Secs . "  0.29 +0100", "%e %b %Y %H.%M %z")->epoch
                            if $Secs =~ /[0-9]+\s.*\s20[0-9]{2}/;
			$Secs = Time::Piece->strptime($Secs . " 0.29 +0100", "%Y-%m-%d %H.%M %z")->epoch
                            if $Secs =~ /20[0-9]{2}\-[0-9]+\-[0-9]+$/;
			# take only recent (3 days from yesterday) values
			next if (not $Secs) || ($Secs < $Recent);
			if ( $Secs > $LastDsecs ) { $max = 0; };
                        $mm_def = 1
                            if (defined $data{$mm_aqi}{$ID}{'avg'}{'index'})
                                && ($data{$mm_aqi}{$ID}{'avg'}{'index'} > 0);
                        my $index = $data{$TBL}{$ID};
                        $index = $data{$mm_aqi}{$ID}{'avg'}{'index'}
                            if $mm_def;
                        # make sure the range is 0 - 10 (e.g. AQI range is 0 - 500)
                        if( $AQItable{$TBL}{range} > 400 ) { # AQI range, special
                            # non linear scale
                            my $new = 0;
                            if ( $data{$TBL}{$ID} <= 50 ) {
                                $new = ($data{$TBL}{$ID} - 0) / (50 - 0) * (3 - 0);
                            } elsif ( $data{$TBL}{$ID} <= 100) {
                                $new = ($data{$TBL}{$ID} - 50) / (100 - 50) * (6 - 3);
                            } elsif ( $data{$TBL}{$ID} <= 150) {
                                $new = ($data{$TBL}{$ID} - 100) / (150 - 100) * (8 - 6);
                            } elsif ( $data{$TBL}{$ID} <= 200) {
                                $new = ($data{$TBL}{$ID} -150) / (200 - 150) * (9 - 8);
                            } elsif ( $data{$TBL}{$ID} <= 300) {
                                $new = ($data{$TBL}{$ID} - 200) / (300 - 200) * (10 - 9);
                            } else {
                                $new = ($data{$TBL}{$ID} - 300) / (600 - 300) * (11 - 10);
                                $new = 11 if $new > 11;
                            }
                            $index = int($new*10+0.5)/10;
                        } else {
                            $index = int(($data{$TBL}{$ID}*10/$AQItable{$TBL}{range}+0.05)*10)/10;
                        }
                        if ( $index > $max ) {
                            my $nr = $ID;
                            $nr = $data{identity}{$ID}
                                    if (defined $data{identity}{$ID})
                                        && $data{identity}{$ID};
                            next if not defined $data{$AQItable{$TBL}{maxGOM}}{$ID};
                            if ( defined $data{location}{$ID} ) {
                                $name = "$data{location}{$ID} met ID $nr";
                                $name = "<a href=\"/?q=$data{identity}{$ID}\">$name</a> "
                                    if (defined $data{identity}{$ID})
                                        && $data{identity}{$ID};
                                $name = "van $name ";
                            }
                            $loc = $data{$AQItable{$TBL}{maxGOM}}{$ID}; $maxID = $ID;
                            $loc = $data{$mm_aqi}{$ID}{'gom'}
                                if $mm_def
                                    && (defined $data{$mm_aqi}{$ID}{'gom'})
                                    && (scalar($data{$mm_aqi}{$ID}{'gom'}) !~ /^HASH/)
				    && $data{$mm_aqi}{$ID}{'gom'};
                            $loc =~ s/(chtt=[^\|]*\+index)\|/$1+$nr|/;
                            # add min max arrow if value provided
                            if ( (defined $data{$mm_aqi}{$ID}{min}{index}) &&
                                (defined $data{$mm_aqi}{$ID}{max}{index}) ) {
                                my $min = int($data{$mm_aqi}{$ID}{min}{index}*100/$AQItable{$TBL}{range}+0.5);
                                my $max = int($data{$mm_aqi}{$ID}{max}{index}*100/$AQItable{$TBL}{range}+0.5);
                                $loc =~ s/chd=(t:[0-9\.]*)\&/chd=$1|$min,$max\&/;
                                $loc =~ s/chls=([0-9,]+)\|([0-9]+)\&/chls=3,10,50\|2,1,5\|$2\|1\&/;
                                $loc =~ s/(chxl=0:\|[^\|]+)\|/$1\|min\|max\|/;
                                $Fnd_one = 1;
                            }
                            $LastDsecs = $Secs; $LastD = $data{$AQItable{$TBL}{maxLD}}{$ID};
                            $max = $index;
                        }
		      }
		    }
                    $loc = ''; # disabled Gom deprecated
                    my $STRG = '';
                    if ( $add > 0 ) {
                        $STRG="
        <p>Voor de beschikbare daggemiddelden van de verschillende luchtvervuilers wordt een z.g. luchtkwaliteits index berekend. Enkele bekende indices:<ul>";
                        $STRG.="<li>de EU <a href=\"http://www.airqualitynow.eu/about_indices_definition.php\">Common Air Quality Index</a> (CAQI) uit 2002, bijgesteld in 2012. Deze index wordt nauwelijks nog gehanteerd.</li>";
                        $STRG.="<li>de internationale Air Quality Index (<a href=\"https://airnow.gov/index.cfm?action=aqibasics.aqi\">AQI</a> voor o.a. de VS en China 1968/2012.";
			$STRG.="<br />De grafiek van de Air Quality Index (AQI) index over de laatste 48 uur in Horst ad Maas wordt hieronder weergegeven."
		       		if ($AQIprint{aqi} > 1) || ($AQIprint{maxaqi} > 1 );
			$STRG.="</li>";
                        $STRG.="<li>de Air Quality Health index <a href=\"https://www.ec.gc.ca/cas-aqhi/default.asp?Lang=En&n=065BE995-1\">AQHI</a> voor Canada 2002/2013). Dit is een samengestelde index over tenminste drie indicatoren.</li>";
			$STRG.="<li>de RIVM LuchtKwaliteitsIndex <a href=\"http://www.rivm.nl/Documenten_en_publicaties/Algemeen_Actueel/Nieuwsberichten/2015/Nieuwe_app_mijn_luchtkwaliteit_gelanceerd\">LKI index</a> (eng. LCI) uit sept 2015.</li>";
                        $STRG.="</ul>Als achtergrondskleur wordt de kleurcode of kenmerking bijv. <em>goed</em> gebruikt van de betreffende index standaard.</p>";
                       $STRG.="<p>De vier indices standaarden verschillen enigzins. Maar in tegenstelling tot de EU normerings regels wordt het aangeven van de luchtkwaliteits wel iets eenvoudiger en beter over de diverse lokaties en landen vergelijkbaar.<br />";
                       $STRG.="Opmerking: De verschillende luchtkwaliteits indices verschillen behalve in de schaal van de index, in de periode (per uur of per dag), samengesteld of de maximale index van enkele stoffen, maar ook in de kenmerking. Wat bij de ene index als <em>gezond</em> gekenmerkt wordt, wordt elders als <em>matig</em> aangegeven (technisch: de index kenmerking is niet lineair). De index wordt berekend uit de gemiddelde dagwaarde metingen. Voor de LKI index is de index daardoor een conservatieve kenmerking.</p>";
		      if ( $loc ) {
                        my $strt = ''; my $end = '';
                        if ( (defined $data{location}{$maxID}) &&
                            $data{location}{$maxID} ) {
                            $end = "</a>";
                            $strt = "<a href=\"/?q=$data{identity}{$maxID}\">";
                        }
                        $STRG.="<p>${strt}<IMG ALT=\"LKI, AQI of AQHI regionale luchtkwaliteits index\" style=\"float:right; margin:5px; width:200px\" src=\"$loc\" />${end}In nevenstaande plaatje weergegeven wordt de hoogste bekende gemiddelde dagwaarde van deze indices in de regio Nrd Limburg (in dit geval ${name}op $LastD) weergegeven.";
                        $STRG.="<br />De minimale en maximale luchtkwaliteits Index van die dag wordt, indien bekend tevens weergegeven in het plaatje."
                            if $Fnd_one;
                        $STRG.="</p>";
                      }
                      $STRG.="<p>Meer informatie over de index voorkomend in onderstaande tabel met meest recente luchtkwaliteits berekeningen:<br /><div class=\"rteindent\">";
		    }
                    PutIt("${STRG}\n$E") if $STRG;
		    PutIt( "<em>AQI index</em> (range 0 t/m 500): In de onderstaande tabellen wordt ook de internationale Air Quality Index (AQI) voor de gemeten luchtvervuilende stoffen aangegeven. De hoogste gemeten AQI index van een of meer gemeten stoffen (O<span style=\"font-size:50%\">3</span>, NO<span style=\"font-size:50%\">2</span>, PM<span style=\"font-size:50%\">10</span>, PM<span style=\"font-size:50%\">2.5</span> SO<span style=\"font-size:50%\">2</span> en CO) in de regio wordt aangegeven.<br />\n$E")
		       if ($AQIprint{aqi} > 1) || ($AQIprint{maxaqi} > 1 );
		    PutIt( "<em>AQHI index</em> (range 0 t/m 10): In de onderstaande tabellen wordt voor zover mogelijk de Canadese Air Quality Health Index (AQHI index) aangegeven. Deze index is een samengestelde waarde van drie indicatoren: O<span style=\"font-size:50%\">3</span>, NO<span style=\"font-size:50%\">2</span> en PM<span style=\"font-size:50%\">2.5</span>. Opmerking: de AQHI indexwaarde is een cumulatieve kenmerking.<br />\n$E")
		       if $AQIprint{aqhi} > 1;
		    PutIt( "<em>CAQI index</em> (range 0 t/m 100): In de onderstaande tabellen wordt de EU Algemene Europese LuchtKwaliteitsIndex (CAQI index) aangegeven. Deze luchtkwaliteits index is de maximale waarde van de individuele CAQI index van O<span style=\"font-size:50%\">3</span>, optioneel NO<span style=\"font-size:50%\">2</span>, PM<span style=\"font-size:50%\">10</span>, (O<span style=\"font-size:50%\">3</span>), en optioneel CO, PM<span style=\"font-size:50%\">2.5</span>, (SO<span style=\"font-size:50%\">2</span>). Er wordt rekening mee gehouden of de lokatie een landelijke omgeving of omgeving is met verkeer Opmerking: de CAQI EU index is oud (2006).<br />\n$E")
		       if $AQIprint{caqi} > 1;
		    PutIt( "<em>LKI index</em> (range 0 t/m 10): In de onderstaande tabellen wordt de RIVM LuchtKwaliteitsIndex (LKI index) aangegeven. Deze luchtkwaliteits index is de maximale waarde van de individuele LKI index voor resp. O<span style=\"font-size:50%\">3</span>, NO<span style=\"font-size:50%\">2</span>, PM<span style=\"font-size:50%\">10</span>, PM<span style=\"font-size:50%\">2.5</span>, en t.z.t. roet. Meer over de LKI index kenmerking is te vinden bij <a href=\"https://www.luchtmeetnet.nl/uitleg#\">www.luchtmeetnet.nl</a>.<br />Opmerking: de LKI index is incompleet en is maar deels een cumulatieve kenmerking: niet alle vier de stoffen worden gemeten en de index wordt eentje opgehoogd indien alle gemeten LKI indices in dezelfde klasse vallen.<br />\n$E")
		       if ($AQIprint{lki} > 1) || ($AQIprint{maxlki} > 1 );
		    PutIt( "</div></p>\n$E");
                    my $howmany = 0;
                    foreach my $aqi_cnt ( 'lki','maxlki','caqi','aqhi','aqi','maxaqi') {
                        $howmany++ if $AQIprint{$aqi_cnt} > 1;
                    }
                    if ( $howmany > 1 ) {
                        # table with chrasteristics, color for an Index def.
                        PutIt( "<p><table style=\"font-size:70%;margin-left:27%\"><caption>Vergelijkingstabel op Index waarde, kenmerking en kleur<br />van enkele luchtkwaliteits Indices</caption><tbody><tr style=\"background-color:#dfdfdf\"><th>schaal</th><th></th><th style=\"min-width:8%\">1</th><th style=\"min-width:8%\">2</th><th style=\"min-width:8%\">3</th><th style=\"min-width:8%\">4</th><th style=\"min-width:8%\">5</th><th style=\"min-width:8%\">6</th><th style=\"min-width:8%\">7</th><th style=\"min-width:8%\">8</th><th>9</th><th>10</th><th>&gt;</th></tr>\n$E");
                        $STRG='';
                        $STRG.="<tr><td style=\"color:#303030;background-color:#afafaf\"><b>AQI</b><td><td style=\"text-align:center;border-right: 2px solid #9f9f9f\" colspan=\"3\">goed</td><td style=\"text-align:center;border-right: 2px solid #9f9f9f\" colspan=\"3\">matig</td><td style=\"text-align:center;border-right: 2px solid #9f9f9f\" colspan=\"2\">opgepast</td><td style=\"text-align:center;border-right: 2px solid #9f9f9f\" colspan=\"2\">ongezond</td><td style=\"text-align:left;border-right: 2px solid #9f9f9f\" colspan=\"1\">gevaar</td></tr><tr><td style=\"text-align:center;color:#303030;background-color:#dfdfdf\">index<td><td style=\"text-align:right;color:".Text_col_schemed('#00e400').";background-color:#00e400;border-right:0px\" colspan=\"3\">&lt;50</td><td style=\"text-align:right;color:".Text_col_schemed('#ffff00').";background-color:#ffff00;border-right:0px\" colspan=\"3\">&lt;100</td><td style=\"text-align:right;color:".Text_col_schemed('#ff7e00').";background-color:#ff7e00;border-right:0px\" colspan=\"2\">&lt;150</td><td style=\"text-align:right;color:".Text_col_schemed('#ff0000').";background-color:#ff0000;border-right:0px\" colspan=\"1\">&lt;200</td><td style=\"text-align:right;color:".Text_col_schemed('#8f3f97').";background-color:#8f3f97;border-right:0px\" colspan=\"1\">&lt;300</td><td style=\"text-align:right;color:".Text_col_schemed('#7e0023').";background-color:#7e0023\" colspan=\"1\">&lt;500</td></tr>"
                        if ($AQIprint{aqi} > 1) || ($AQIprint{maxaqi} > 1 );
                        $STRG.="<tr><td style=\"color:#303030;background-color:#afafaf\"><b>AQHI</b><td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">laag</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">matig</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">hoog</td><td style=\"text-align:left;border-right:2px solid #9f9f9f\" colspan=\"2\">zeer hoog</td></tr><tr><td style=\"text-align:center;color:#303030;background-color:#dfdfdf\">index<td><td style=\"text-align:center;color:".Text_col_schemed('#00ccff').";background-color:#00ccff;border-right:0px\">1</td><td style=\"text-align:center;color:".Text_col_schemed('#0099cc').";background-color:#0099cc;border-right:0px\">2</td><td style=\"text-align:center;color:".Text_col_schemed('#006699').";background-color:#006699;border-right:0px\">3</td><td style=\"text-align:center;color:".Text_col_schemed('#ffff00').";background-color:#ffff00;border-right:0px\">4</td><td style=\"text-align:center;color:".Text_col_schemed('#ffcc00').";background-color:#ffcc00;border-right:0px\">5</td><td style=\"text-align:center;color:".Text_col_schemed('#ff9933').";background-color:#ff9933;border-right:0px\">6</td><td style=\"text-align:center;color:".Text_col_schemed('#ff6666').";background-color:#ff6666;border-right:0px\">7</td><td style=\"text-align:center;color:".Text_col_schemed('#ff0000').";background-color:#ff0000;border-right:0px\">8</td><td style=\"text-align:center;color:".Text_col_schemed('#cc0000').";background-color:#cc0000;border-right:0px\">9</td><td style=\"text-align:center;color:".Text_col_schemed('#990000').";background-color:#990000;border-right:0px\">10</td><td style=\"text-align:center;color:".Text_col_schemed('#660000').";background-color:#660000\">11</td></tr>"
                        if $AQIprint{aqhi} > 1;
                        $STRG.="<tr><td style=\"color:#303030;background-color:#afafaf\"><b>CAQI</b><td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">zeer laag</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">laag</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"2\">risico</td><td style=\"text-align:left;border-right:2px solid #9f9f9f\" colspan=\"2\">hoog</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"1\"></td></tr><tr><td style=\"text-align:center;color:#303030;background-color:#dfdfdf\">index<td><td style=\"text-align:right;color:".Text_col_schemed('#79bc6a').";background-color:#79bc6a;border-right:0px\" colspan=\"3\">&lt;25</td><td style=\"text-align:right;color:".Text_col_schemed('#b9ce45').";background-color:#b9ce45;border-right:0px\" colspan=\"2\">&lt;50</td><td style=\"text-align:right;color:".Text_col_schemed('#edc100').";background-color:#edc100;border-right:0px\" colspan=\"2\">&lt;75</td><td style=\"text-align:right;color:".Text_col_schemed('#f69208').";background-color:#f69208;border-right:0px\" colspan=\"2\">&lt;100</td><td style=\"text-align:right;color:".Text_col_schemed('#f03667').";background-color:#f03667;border-right:0px\" colspan=\"2\">&lt;125</td></tr>"
                        if $AQIprint{caqi} > 1;
                        $STRG.="<tr><td style=\"color:#303030;background-color:#afafaf\"><b>LKI</b><td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">goed</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"3\">matig</td><td style=\"text-align:center;border-right:2px solid #9f9f9f\" colspan=\"2\">onvoldoende</td><td style=\"text-align:left;border-right:2px solid #9f9f9f\" colspan=\"3\">slecht</td></tr><tr><td style=\"text-align:center;color:#303030;background-color:#dfdfdf\">index<td><td style=\"text-align:center;color:".Text_col_schemed('#002bf7').";background-color:#002bf7;border-right:0px\">1</td><td style=\"text-align:center;color:".Text_col_schemed('#009cf9').";background-color:#009cf9;border-right:0px\">2</td><td style=\"text-align:center;color:".Text_col_schemed('#c4ecfd').";background-color:#c4ecfd;border-right:0px\">3</td><td style=\"text-align:center;color:".Text_col_schemed('#fffda4').";background-color:#fffda4;border-right:0px\">4</td><td style=\"text-align:center;color:".Text_col_schemed('#fffc4d').";background-color:#fffc4d;border-right:0px\">5</td><td style=\"text-align:center;color:".Text_col_schemed('#f4e645').";background-color:#f4e645;border-right:0px\">6</td><td style=\"text-align:center;color:".Text_col_schemed('#ff9845').";background-color:#ff9845;border-right:0px\">7</td><td style=\"text-align:center;color:".Text_col_schemed('#fe7626').";background-color:#fe7626;border-right:0px\">8</td><td style=\"text-align:center;color:".Text_col_schemed('#ff0a17').";background-color:#ff0a17;border-right:0px\">9</td><td style=\"text-align:center;color:".Text_col_schemed('#dc0610').";background-color:#dc0610;border-right:0px\">10</td><td style=\"text-align:center;color:".Text_col_schemed('#a21794').";background-color:#a21794;border-right:0px\">11</td></tr>"
                        if ($AQIprint{lki} > 1) || ($AQIprint{maxlki} > 1);
                        PutIt( $STRG . "</tbody></table></p>\n$E" );
                    }
                }
                # add an AQI Google meter image? eg  ...abc AQHIIMAGE HadM abc...
		if( /\s(AQI|AQHI|LKI|CAQI)IMAGE\s+([A-Za-z0-9]+)\s/ ) {
		    my $table = $2; my $t = lc($1);
# TO DO add styling!
                    $table = "<img src=\"$data{$t.'gom'}{$table}\" /> $table"
			if defined $data{$t.'gom'}{$table};
		    s/\s(AQI|AQHI|LKI|CAQI)IMAGE\s+[A-Za-z0-9]+\s/ /;
		}
		if ( /[a-z]{2}\s+[0-9]{1,2}\s+[a-z]{3,}\s+20[0-9]{2}/i ) {
		    s/[a-z]{2}\s+[0-9]{1,2}\s+[a-z]{3,}\s+20[0-9]{2}/$data{date}{YESTERDAY}/i if not $no_table;
		    s/[a-z]{2}\s+[0-9]{1,2}\s+[a-z]{3,}\s+20[0-9]{2}/$data{date}{TODAY}/i if $no_table;
		# table with mnenomics table IDs
		} elsif ( /<table.*\sid="afkortingen"/ ) {
                    s/id="afkortingen"/title="Door te clicken op de afkorting worden meer details van het station getoond" id="afkortingen"/;
		    PutIt $_;
		    my $glue = "\n"; $glue = "" if not $T;
		    while( $_ = GetIt ) {
		 	last if not $_;
			PutIt $_ if not /<\/tr>/; last if /<\/tr>/;
		    }
		    PutIt "$T$T</tr>$E$glue";
		    for my $ID (@stationID) {
			$_ = $data{name}{$ID}; s/meetstation\s*//;
			s/\s\([^)]*\)//;
                        my $Alias = $ID;
                        if ( defined $data{identity}{$ID} ) {
                            $Alias = "<a href=\"/?q=$data{identity}{$ID}\">$ID</a>"
                                if $data{identity}{$ID} !~ /NULL/ ;
                        }
			PutIt "$T$T<tr>$E$glue$T$T$T<td>&nbsp;$Alias</td>$E\n$T$T$T<td>&nbsp;" . $_ .
				"</td>$E\n$T$T</tr>$E$glue";
		    }
		    while( $_ = GetIt ) {
			last if not $_ ; last if /<\/tbody>/ ;
		    }
		##### table with PM values for last day, current year, year average
		} elsif ( /<table\s.*\sid="PMdagwaarden"/ ) {
		    my $width = ""; my @NRpolsa = split /\|/, $scan_only;
		    my $NRpols = $#NRpolsa + 1; undef @NRpolsa;
		    PutIt $_;
		    while( $_ = GetIt ) {
			last if not $_;
			if ( /\s[a-z]{2}\s+[0-9]+\s+[a-z]{3}\s+20[0-9][0-9]/ ) {
			   s/\s[a-z]{2}\s+[0-9]+\s+[a-z]{3}\s+20[0-9][0-9]/ $data{date}{YESTERDAY}/;
			}
			PutIt $_; last if /<\/td>/;
		    }
		    last if not $_;

		    # station names
		    for my $ID (@stationID) {
			PutIt "$T$T$T<td class=\"rtecenter\" colspan=\"$NRpols\" rowspan=\"1\" align=\"center\" style=\"border-left:3px solid white;background-color:rgb(204, 204, 204); border-color:rgb(255, 255, 255)\"><strong>$ID</strong></td>$E\n";
		    }
		    $_ = SKIP2TD; last if not $_;

		    # sensor names
		    for my $ID (@stationID) {
		        my $lb = "border-left:3px solid white;";
                        for my $pol (@sensor) {
			    next if $pol !~ /$scan_only/;
                            # PM??yr year average
                            my $p = uc($pol); $p =~ s/_//;
			    if ( $p =~ /([A-Z]+)([0-9]+)/ ) {
			        $p = "$1<span style=\"font-size:85%\">$2</span>";
				$p =~ s/25/2.5/ if $p =~ /PM.*25/;
			    }
			    # do not print header if none values was measured
			    $width = "";
			    $p = "" if (not defined $data{avg}{$ID}{$pol})
                                    || (not $data{avg}{$ID}{$pol});
			    $width = " width=0px"
                                if (not defined $data{avg}{$ID}{$pol})
                                    || (not $data{avg}{$ID}{$pol});
                            PutIt "$T$T$T<td class=\"rtecenter\" style=\"${lb}background-color:rgb(221, 221, 221); border-color:rgb(255, 255, 255)\"$width><em>$p</em></td>$E\n";
			    $lb = "";
                        }
                    }
		    $_ = SKIP2TD; last if not $_;

		    # per station last 365 days average values
		    for my $ID (@stationID) {
		        my $lb = "style=\"border-left:3px solid white\"";
			for my $pol (@sensor) {
			    next if $pol !~ /$scan_only/i;
			    my $col = $data{avg}{$ID}{$pol};
			    my $ugm3 = ""; # $ugm3 = "<br />$E\n$T$T${T}µg/m³";
		 	    $col = "<span style=\"color:#FF0000\">$col</span>" if $data{avg}{$ID}{$pol} =~ /!/;	
			    $col =~ s/!//; $width = "";
			    $ugm3 = "" if not $col;
			    $col = "" if not $col;
			    $width = "";
			    $width = " width=0px" if ($stations{$ID}{$pol} < 0);
			    PutIt "$T$T$T<td class=\"rtecenter\" $lb$width><strong>$col</strong>$ugm3</td>$E\n";
			    $lb = "";
			}
		    }
		    $_ = SKIP2TD; last if not $_;

		    # number of days exceeding limits per station
		    for my $ID (@stationID) {
		        my $lb = "style=\"border-left:3px solid white\"";
			for my $pol (@sensor) {
			    next if $pol !~ /$scan_only/i;
			    my $col = $data{days}{$ID}{$pol}; $col =~ s/!//;
		 	    $col = "<span style=\"color:#FF0000\">$col</span>" if $data{days}{$ID}{$pol} =~ /!/;	
			    # space if no measurements where done for this sensor
			    $col = "" if (not defined $data{avg}{$ID}{$pol})
                                || (not $data{avg}{$ID}{$pol});
			    $col = "&#8805;$data{norm}{MAX}{$pol}<br /><strong>$col</strong>"
				if $col;
			    my $width = "";
			    $width = " width=0px" if not $col;
			    PutIt "$T$T$T<td class=\"rtecenter\" $lb$width>$col</td>$E\n";
			    $lb = "";
			}
		    }
		    $_ = SKIP2TD; last if not $_;

		    # last day average per station
		    for my $ID (@stationID) {
		        my $lb = "border-left:3px solid white;";
			for my $pol (@sensor) {
			    next if $pol !~ /$scan_only/;
			    my $diff = $data{diff}{$ID}{$pol};
			    my $ColDiff = ""; my $col = 0;
			    $col = $data{today}{$ID}{$pol}
				if defined $data{today}{$ID}{$pol};
			    $col =~ s/!//;
			    my $ugm3 = ""; # $ugm3 = "<br />$E\n$T$T${T}µg/m³";
			    if ( defined $data{diff}{$ID}{$pol} ) {
			      $diff = '<br />▲<span style="font-size:80%">+' . $diff . '</span>'
				if $data{diff}{$ID}{$pol} > 0;
			      $diff = '<br />▼<span style="font-size:80%">' . $diff .'</span>'
				if $data{diff}{$ID}{$pol} < 0;
			      $diff = '<br />♦' if not $data{diff}{$ID}{$pol};
			      $ColDiff = ";color:#FF0000" if $data{diff}{$ID}{$pol} > 5;
			      $ColDiff = ";color:#006300" if $data{diff}{$ID}{$pol} < -5;
			      $diff = "<span style=\"font-size:90%$ColDiff\">$diff</span>"
				if $col > 1;
			    }
		 	    $col = "<span style=\"color:#FF0000\">$col</span>"
				if ($col > 1) && ($data{today}{$ID}{$pol} =~ /!/);	
			    if( (defined $data{lastday}{$ID}{$pol})
			        && $data{lastday}{$ID}{$pol}
			        && ($data{lastday}{$ID}{$pol} ne $data{date}{YESTERDAY})
				) {
			        $col = "?"; $ugm3 = ""; $diff = "";
			    }
			    do { $ugm3 = ""; $diff = ""; $col = ""; }
				if not $col;
			    $width = "";
			    $width = " width=0px"
				if ($stations{$ID}{$pol} < 0);
			    PutIt "$T$T$T<td class=\"rtecenter\" style=\"${lb}background-color:rgb(225,225,225);\"$width><strong>$col</strong>$diff$ugm3</td>$E\n";
			    $lb = "";
			}
		    }
                    foreach my $TBL ( sort { $AQItable{$a}{pri} <=> $AQItable{$b}{pri} } keys %AQItable ) {
		    if ( $AQIprint{$TBL} > 1 ) { # aqi table is activated
			my $dag = $data{date}{YESTERDAY};
                        my $mm_aqi = 'mm-'.$TBL; $mm_aqi =~ s/max//i;
			$dag =~ s/^[a-z]{2}\s+//; 
                        my $this_aqi = "<strong>".uc($TBL)."</strong>";
                        $this_aqi =~ s/MAX//i 
                            if $this_aqi =~ /max/i;
                        #$this_aqi = " en<br />hoogste $this_aqi"
                        #    if $AQItable{$TBL}{maxAQ} =~ /max/;
                        my $fnd_one = '';
                        for my $ID (@stationID) {
                            $fnd_one = "<br \>daggemiddelde (min .. max)"
                                if (defined $data{$mm_aqi}{$ID}{'min'}{'index'})
                                    && (defined $data{$mm_aqi}{$ID}{'max'}{'index'})
                                    && ($data{$mm_aqi}{$ID}{'min'}{'index'} > 0)
                                    && ($data{$mm_aqi}{$ID}{'max'}{'index'} > 0);
                            last if $fnd_one;
                        }
                        $this_aqi .=  $fnd_one;
                        my $extra = '';
                        $extra = join ',', @{$AQItable{$TBL}{using}};
                        $extra =~ s/([A-Z])([0-9\.]+)/$1<span style="font-size:50%">$2<\/span>/g;
                        $extra = "<br /><span style=\"font-size=80%\">($extra)</span>" if $extra;
		        PutIt "$E\n$T</tr>$E\n$T<tr>$E\n$T$T<td class=\"rtecenter\" colspan=\"1\" rowspan=\"1\" style=\"border-left:3px solid white;background-color:rgb(204, 204, 204); border-color:rgb(255, 255, 255)\">laatst bekende ".$this_aqi."$extra</td>$E\n";
			for my $ID (@stationID) {
                            my $lb = "border-left:3px solid white;";
			    for my $pol (@sensor) {
			        next if $pol !~ /$scan_only/;
			        my $col = "rgb(221, 221, 221)"; my $tcol = '';
                                my $aqi = "";
			        my $width = "";
				$aqi = $data{$TBL}{$ID}
				    if (defined $data{$TBL}{$ID}) && ($TBL !~ /^aqi/);
				$aqi = $data{$TBL}{$ID}{$pol}
				    if ($TBL =~ /^aqi/)
                                        && (defined $data{$TBL}{$ID}{$pol})
                                        && $data{$TBL}{$ID}{$pol};
                                if ( $aqi ) {
                                    my $mm_def = 0; my $qual = '';
                                    $mm_def = 1
                                       if (defined $data{$mm_aqi}{$ID}{'avg'}{'index'} )
                                            && $data{$mm_aqi}{$ID}{'avg'}{'index'} > 0;
                                    if ( $mm_def ) {
                                        my $better = $data{$mm_aqi}{$ID}{'avg'}{'index'};
                                        $better = int($better*10+0.5)/10
                                            if $better < 20;
                                        $better = int($better+0.5)
                                            if $better > 20;
                                        $mm_def = 0 if $better < $aqi;
                                        $aqi = $better if $better > $aqi;
                                    }
                                    $aqi = "<em>$aqi</em>";
                                    $aqi = "$aqi<br \>" .
                                        sprintf("<span style=\"font-size:70%%\">(%d .. %d)</span>",
                                            int($data{$mm_aqi}{$ID}{'min'}{'index'} + 0.5),
                                            int($data{$mm_aqi}{$ID}{'max'}{'index'} + 0.5))
                                        if (defined $data{$mm_aqi}{$ID}{'min'}{'index'})
                                            && (defined $data{$mm_aqi}{$ID}{'max'}{'index'})
                                            && ($data{$mm_aqi}{$ID}{'min'}{'index'} > 0)
                                            && ($data{$mm_aqi}{$ID}{'max'}{'index'} > 0);
				    $aqi = "$aqi<br /><span style=\"font-size:70%\">$data{$AQItable{$TBL}{maxAQ}}{$ID}</span>"
					if (not $TBL =~ /^aqi/) &&
                                           (defined $data{$AQItable{$TBL}{maxAQ}}{$ID})
					   && $data{$AQItable{$TBL}{maxAQ}}{$ID};
				    $qual = "$data{aqiqual}{$ID}{$pol}"
					if ($TBL =~ /^aqi/) &&
                                           (defined $data{aqiqual}{$ID}{$pol}) &&
					   $data{aqiqual}{$ID}{$pol};
                                    $qual = $data{$mm_aqi}{$ID}{'avg'}{'qual'}
                                        if $mm_def
                                           && (defined $data{$mm_aqi}{$ID}{'avg'}{'qual'})
                                           && $data{$mm_aqi}{$ID}{'avg'}{'qual'};
                                    $aqi .= "<br /><span style=\"font-size:70%\">$qual</span>"
                                        if $qual;
				    $col = $data{$AQItable{$TBL}{colorAQ}}{$ID}
				        if defined $data{$AQItable{$TBL}{colorAQ}}{$ID};
				    $col = $data{aqicolor}{$ID}{$pol}
				        if ($TBL =~ /^aqi/) &&
                                            (defined $data{aqicolor}{$ID}{$pol});
                                    $col = $data{$mm_aqi}{$ID}{'col'}
                                        if $mm_def
                                            && (defined $data{$mm_aqi}{$ID}{'col'})
                                            && $data{$mm_aqi}{$ID}{'col'};
                                    
				    $col =~ s/0x/#/;
                                    $tcol = 'color:'.Text_col_schemed($col).';';
                                } else {
			            $width = " width=0px"; $aqi = "";
				}
                                PutIt "$T$T$T<td class=\"rtecenter\" colspan=\"$NRpols\" align=center style=\"${lb}background-color:$col;${tcol}border-color:rgb(255, 255, 255)\"$width>$aqi</td>$E\n";
			        $lb = "";
                                last;
			    }
                        }
                      }
		    }

		    while( $_ = GetIt ) { last if not $_; last if /<\/tr>/ ; }
		    last if not $_; 

                } elsif ( (/Bezoek de website van AQICN/) && $forecast ) {
                    # forecast weather and PM2.5 table insert via FILE=
                    # delete the refs to AQICN forecast
                    while( $_ = GetIt ) { last if /Onderstaand wordt in de tabellen/ };
#                    s/.*//;
#                    my $STRG='<p>';
#                    $STRG = "<table style='color:black;background:#f9f9f9;border-radius:7px;-moz-border-radius:7px;-webkit-border-radius:7px;box-shadow: 0px 7px 10px 0px #a0a0a0;text-align:left;padding:0px;padding-top:3px;padding-bottom:0.3%;margin:0px;border-spacing:0px;border:0px solid black;width:100%;'>$E\n";
#                    $STRG .= "<caption valign=top align=center><span style='color:#0376ab'>Verwachting fijnstof <span style='font-size:70%'>2.5</span> (<a href='https://aqicn.org/city/netherland/horst-a/d-maas/hoogheide/' title='more local forecasts weather and Air Quality Index'>AQICN</a>) en weer (<a href='https://www.yr.no/place/Netherlands/Limburg/Grubbenvorst/' title='more local weather information'>YR.no</a>) voor komende 48 uur voor Horst a/d Maas</span></caption>$E\n";
#                    $STRG .= "<tbody><tr><td><div id='${forecast}' style='width: 100%; height: 310px; margin: 0 auto'>PM2.5 en weersverwachting voor komende 48 uur</div></td></tr></tbody></table></p><p>$E\n";
#                    $STRG .= "<a href='http://aqicn.org/' title='Bezoek de website van AQICN'><img alt='logo AQICN' src='http://aqicn.org/images/logo/regular.png' style='float:left; height:50px; margin:5px; width:50px' /></a>$E\n";
#                    PutIt $STRG;

		### specific days exceeding day average limit
		} elsif ( /<table\s.*id="jaaroverschrijdingen"/ ) {
		    my $width;
		    PutIt $_;
		    # header with station names
		    while( $_ = GetIt ) {
			last if not $_;
                        PutIt $_; last if /<\/td>/ ;
                    }
		    last if not $_;
		    for my $ID (@stationID) {
		        my $lb = "border-left:3px solid white;";
			PutIt "$T$T$T<td class=\"rtecenter\" colspan=\"$NRpols\" rowspan=\"1\" align=\"center\" style=\"${lb}background-color:rgb(204, 204, 204)\"><strong>&nbsp;$ID</strong></td>$E\n";
		    }
		    while( $_ = GetIt ) { last if not $_; last if /<\/tr>/; }
		    last if not $_; PutIt $_;
		    if ( not /<\/td>/ ) {
		        while( $_ = GetIt ) {
			    last if not $_;
                            PutIt $_; last if /<\/td>/ ;
                        }
		        last if not $_;
		    }
		    # sensors per station
		    for my $ID (@stationID) {
		        my $lb = "border-left:3px solid white;";
			for my $pol (@sensor) {
                            next if $pol !~ /$scan_only/;
                            my $p = uc($pol); $p =~ s/_//;
                            if ( $p =~ /([A-Z]+)([0-9]+)/ ) {

                                $p = "$1<span style=\"font-size:85%\">$2</span>";
				$p =~ s/25/2.5/ if $p =~ /PM.*25/;
                            }
			    $width = " width=50px";
			    # do not print header if none values was measured
			    $p = "" if (not defined $data{avg}{$ID}{$pol})
                                || (not $data{avg}{$ID}{$pol});
			    $width = " width=0px"
                                if (not defined $data{avg}{$ID}{$pol})
                                    || (not $data{avg}{$ID}{$pol});
                            PutIt "$T$T$T<td class=\"rtecenter\" style=\"${lb}background-color:rgb(204, 204, 204)\"$width><em>$p</em></td>$E\n";
			    $lb = "";
                        }
		    }
		    while( $_ = GetIt ) { last if not $_; last if /<\/tr>/; }
		    last if not $_; PutIt $_;
		    if ( not /<\/td>/ ) {
		        while( $_ = GetIt ) { last if not $_;
			    PutIt $_; last if /<\/td>/;
			}
		        last if not $_;
		    }
		    # days exceeding limit per sensor and station
                    for my $ID (@stationID) {
		        my $lb = "border-left:3px solid white;";
                        for(my $i=0; $i <= $#sensor; $i++) {
			    my $pol = $sensor[$i];
			    next if  $pol !~ /$scan_only/;
			    $width = " width=50px";
                            $col = "";
			    foreach my $datum (keys %data) {
				next if $datum !~ /20[0-9][0-9]-[0-1][0-9]-[0-3][0-9]/;
				next if not defined $data{$datum}{$ID};
			        if( defined $data{$datum}{$ID}[$i] ) {
				   $col = 0
					if ($data{$datum}{$ID}[$i] =~ /!/)
                                            && (not $col);
				   $col++
					if ($data{$datum}{$ID}[$i] =~ /!/);
				}
			    }
                            $col = "<span style=\"color:#FF0000\">$col</span>"
				if $col && ($col >= $data{norm}{MAXdays}{$pol});
			    $width = " width=0px"
				if (not defined $data{avg}{$ID}{$pol})
                                    || (not $data{avg}{$ID}{$pol});
			    $col = "" if (not defined $data{avg}{$ID}{$pol})
                                    || (not $data{avg}{$ID}{$pol});
                            PutIt "$T$T$T<td class=\"rtecenter\" style=\"${lb}background-color:rgb(230, 230, 230)\"$width><strong>$col</strong></td>$E\n";
			    $lb = "";
                        }
                    }
		    my $glue = "\n"; $glue = "" if not $T;
		    while( $_ = GetIt ) { last if not $_; last if /<\/tr>/; }
                    last if not $_; PutIt "<\/td>";
		    # print days exceeding limit
		    my $row = 0;
		    foreach my $datum (sort { $b cmp $a ; } keys %data) {
                        next if $datum !~ /^20[0-9]{2}-[02][0-9]-[0-3][0-9]$/ ;
			$row++; $row %= 2;
			PutIt "$T$T<tr>$E$glue";
                        PutIt "$T$T$T<td class=\"rteright\" style=\"padding-right:10px;background-color:rgb(238,238,238);\">" .
			    NL_time($datum) ."</td>$E\n" unless $row;
                        PutIt "$T$T$T<td class=\"rteright\" style=\"padding-right:10px\">" .
			    NL_time($datum) ."</td>$E\n" if $row;
                        for my $ID (@stationID) {
		            my $lb = "border-left:3px solid white;";
		            $lb = "border-left:2px solid rgb(245,245,245);" if $row;
			    for ( my $i = 0; $i <= $#sensor; $i++ ) {
				next if $sensor[$i] !~ /$scan_only/;
			        if( (not defined $data{$datum}{$ID})
                                    || (not defined $data{$datum}{$ID}[$i])
                                    || (not $data{$datum}{$ID}[$i])
                                ) {
				  # it is not defined, try to collect it
				  # this action should have been done by Complete Data
			          # if column define calculate day avg, if not -1
                                  # is the sensor in the DB?
                                  $data{$key}{$ID}[$i] = -1;
                                  $stations{$ID}{$sensor[$i]} = -1
                                    if not defined $stations{$ID}{$sensor[$i]};
                                  my $qry = query('DATA',
                                      "SELECT count(*)
                                       FROM information_schema.columns
                                       WHERE table_name = '$data{table}{$ID}'
                                    AND column_name = '$sensor[$i]'");
                          	  if( (defined $qry) && ($#{$qry} >= 0)
                                    && ($#{$qry->[0]} >= 0)
                                    && ($qry->[0][0] > 0)
                                    ) {  # yes sensor is present
                                      $data{$key}{$ID}[$i] = 0;
                                      $qry = query ('DATA',
                                            "SELECT round(avg($sensor[$i]))
                                              FROM $data{table}{$ID}
                                              WHERE date(datum) = \"$key\" ");
                                      if( (defined $qry) && ($#{$qry} >= 0)
                                        && ($#{$qry->[0]} >= 0) ) {
                                          if( defined $qry->[0][0] ) {
                                             $data{$key}{$ID}[$i] = $qry->[0][0];
                                          } else {
                                            $data{$key}{$ID}[$i] = 0;
                                          }
				      }
				  }
				}
			        my $bg = "";
				$bg = " style=\"${lb}background-color:rgb(238,238,238);\"" if not $row;
			        $bg = " style=\"$lb\"" if $row && $lb;
				$lb = "";
				my $col = $data{$datum}{$ID}[$i];
				$col =~ s/[!\-]//;
                                $col = "<span style=\"color:#FF0000\">$col</span>"
				    if $col && ($col >= $data{norm}{MAX}{$sensor[$i]});
                                $col = "-" if not $data{$datum}{$ID}[$i];
                                $col = "" if (not defined $data{avg}{$ID}{$sensor[$i]}) || (not $data{avg}{$ID}{$sensor[$i]});
				$width = " width=50px";
				$width = " width=0px"
				   if (not defined $data{avg}{$ID}{$sensor[$i]}) || (not $data{avg}{$ID}{$sensor[$i]});
				PutIt "$T$T$T<td class=\"rtecenter\"$bg$width>$col</td>$E\n";
                            }
                        }
			PutIt "$T$T</tr>$E$glue";
		    }
		    while( $_ = GetIt ) { last if not $_; last if /<\/tbody>/; }
		#### change dates to mark changes (optional)
		#} elsif ( /\s+\[(changed|revision_timestamp)\]\s=>\s([0-9]+)/ ) {
		#	my $time = time; s/[0-9]+/$time/;
		}
		last if not $_; PutIt $_;
	    }	    
	    # PutIt(sprintf("%d",time),"changed");	# change node time stamp
	    # PutIt(sprintf("%d",time),"revision_timestamp");
	    return Close_JSON($output);
	}

	# ARGV[0]: data values eg measurements values per sensor; deflt stdin
        $ARGV[0] = "/dev/stdin" if not $ARGV[0];
	# ARGV[1]: node number dflt $Drupal_node
	$Drupal_node = "3911" if not $ARGV[1];	# default node number
        $ARGV[1] = "$Drupal_node" if not $ARGV[1];
	# ARGV[2]: output file name or node number; default $Drupal_node
	# keys: -d debug and -j json output dftl update via drush: drupal sql update
	Get_defs($ARGV[0]) ;  # import data from file in arg1
        
	require "/webdata/luchtmetingen/AQI.pl" if $debug || ($ARGV[1] =~ /print/);
        Print_Table() if $debug;
	if( ($ARGV[1] =~ /print/) ) {
	    shift @ARGV; shift @ARGV;
	    Print_Table( @ARGV );
            exit(0);
        }
	# update node via template originating from that node ($Drupal_node)
	# get input file from argv[1], process it and write it to argv[2]
	exit(0) if $#ARGV < 1;
        $ARGV[2] = $ARGV[1] if $#ARGV < 2;
	Parse_Template($ARGV[1], $ARGV[2]) if $ARGV[1];
	exit 0;
